from cone.app import testing
from cone.app.browser.actions import LinkAction
from cone.app.browser.referencebrowser import ActionAddReference
from cone.app.browser.referencebrowser import ActionRemoveReference
from cone.app.browser.referencebrowser import ReferencableChildrenLink
from cone.app.browser.referencebrowser import ReferenceAction
from cone.app.browser.referencebrowser import ReferenceBrowserModelMixin
from cone.app.browser.referencebrowser import ReferenceListing
from cone.app.interfaces import INavigationLeaf
from cone.app.model import BaseNode
from cone.tile import render_tile
from cone.tile.tests import TileTestCase
from datetime import datetime
from datetime import timedelta
from node.behaviors import UUIDAware
from node.utils import UNSET
from plumber import plumbing
from pyramid.httpexceptions import HTTPForbidden
from yafowil.base import ExtractionError
from yafowil.base import factory
from zope.interface import implementer


@plumbing(UUIDAware)
class RefNode(BaseNode):
    node_info_name = 'ref_node'


class TestBrowserReferenceBrowser(TileTestCase):
    layer = testing.security

    def test_reference_render(self):
        self.layer.new_request()
        widget = factory('reference', name='ref')
        self.checkOutput("""
        <span
          ajax:target="http://example.com/?navigable=&referencable=&root=/&selected=">
          <input class="form-control referencebrowser"
                 id="input-ref" name="ref" readonly="readonly"
                 type="text" value="" />
          <input name="ref.uid" type="hidden" value="" />
          <span class="referencebrowser_trigger" data-reference-name='ref'>
            <i class="ion-android-share">
            </i>
          browse</span>
        </span>
        """, '>\n'.join(widget().split('>')))

        widget.attrs['multivalued'] = True
        self.checkOutput("""
        <span
          ajax:target="http://example.com/?navigable=&referencable=&root=/&selected=">
          <input id="exists-ref" name="ref-exists" type="hidden" value="exists" />
          <select class="form-control referencebrowser"
                  id="input-ref" multiple="multiple" name="ref">
          </select>
          <span class="referencebrowser_trigger" data-reference-name='ref'>
            <i class="ion-android-share">
            </i>
            browse</span>
        </span>
        """, '>\n'.join(widget().split('>')))

    def test_reference_root(self):
        self.layer.new_request()
        widget = factory('reference', name='ref')
        expected = (
            'http://example.com/'
            '?navigable=&referencable=&root=/&selected='
        )
        self.assertTrue(widget().find(expected) > -1)

        widget.attrs['root'] = '/container'
        expected = (
            'http://example.com/'
            '?navigable=&referencable=&root=/container&selected='
        )
        self.assertTrue(widget().find(expected) > -1)

        def root_callable(widget, data):
            return '/computed_root'

        widget.attrs['root'] = root_callable
        expected = (
            'http://example.com/'
            '?navigable=&referencable=&root=/computed_root&selected='
        )
        self.assertTrue(widget().find(expected) > -1)

    def test_reference_referencable(self):
        self.layer.new_request()
        widget = factory('reference', name='ref')
        expected = (
            'http://example.com/'
            '?navigable=&referencable=&root=/&selected='
        )
        self.assertTrue(widget().find(expected) > -1)

        widget.attrs['referencable'] = 'foo'
        expected = (
            'http://example.com/'
            '?navigable=&referencable=foo&root=/&selected='
        )
        self.assertTrue(widget().find(expected) > -1)

        widget.attrs['referencable'] = ['foo', 'bar']
        expected = (
            'http://example.com/'
            '?navigable=&referencable=foo,bar&root=/&selected='
        )
        self.assertTrue(widget().find(expected) > -1)

        widget.attrs['referencable'] = 'foo,bar'
        expected = (
            'http://example.com/'
            '?navigable=&referencable=foo,bar&root=/&selected='
        )
        self.assertTrue(widget().find(expected) > -1)

        def referencable_callable(widget, data):
            return 'computed'

        widget.attrs['referencable'] = referencable_callable
        expected = (
            'http://example.com/'
            '?navigable=&referencable=computed&root=/&selected='
        )
        self.assertTrue(widget().find(expected) > -1)

    def test_reference_navigable(self):
        self.layer.new_request()
        widget = factory('reference', name='ref')
        expected = (
            'http://example.com/'
            '?navigable=&referencable=&root=/&selected='
        )
        self.assertTrue(widget().find(expected) > -1)

        widget.attrs['navigable'] = 'foo'
        expected = (
            'http://example.com/'
            '?navigable=foo&referencable=&root=/&selected='
        )
        self.assertTrue(widget().find(expected) > -1)

        widget.attrs['navigable'] = ['foo', 'bar']
        expected = (
            'http://example.com/'
            '?navigable=foo,bar&referencable=&root=/&selected='
        )
        self.assertTrue(widget().find(expected) > -1)

        widget.attrs['navigable'] = 'foo,bar'
        expected = (
            'http://example.com/'
            '?navigable=foo,bar&referencable=&root=/&selected='
        )
        self.assertTrue(widget().find(expected) > -1)

        def navigable_callable(widget, data):
            return 'computed'

        widget.attrs['navigable'] = navigable_callable
        expected = (
            'http://example.com/'
            '?navigable=computed&referencable=&root=/&selected='
        )
        self.assertTrue(widget().find(expected) > -1)

    def test_reference_target(self):
        self.layer.new_request()
        widget = factory('reference', name='ref')
        expected = (
            'ajax:target="http://example.com/'
            '?navigable=&referencable=&root=/&selected="'
        )
        self.assertTrue(widget().find(expected) > -1)

        widget.attrs['target'] = 'http://domain.com/'
        expected = (
            'ajax:target="http://domain.com/'
            '?navigable=&referencable=&root=/&selected="'
        )
        self.assertTrue(widget().find(expected) > -1)

        def target_callable(widget, data):
            return 'http://computed.com/'

        widget.attrs['target'] = target_callable
        expected = (
            'ajax:target="http://computed.com/'
            '?navigable=&referencable=&root=/&selected="'
        )
        self.assertTrue(widget().find(expected) > -1)

    def test_reference_selected(self):
        request = self.layer.new_request()

        widget = factory('reference', name='ref')
        data = widget.extract(request)
        expected = (
            'ajax:target="http://example.com/'
            '?navigable=&referencable=&root=/&selected="'
        )
        self.assertTrue(widget(data=data).find(expected) > -1)

        widget.getter = ['3604702a-b177-42de-8694-d81e2d993633', 'Label']
        data = widget.extract(request)
        expected = (
            'ajax:target="http://example.com/'
            '?navigable=&referencable=&root=/'
            '&selected=3604702a-b177-42de-8694-d81e2d993633"'
        )
        self.assertTrue(widget(data=data).find(expected) > -1)

        request.params['ref'] = 'Item'
        request.params['ref.uid'] = '745dd2ed-74ad-449b-b55b-b240709b2b0e'
        data = widget.extract(request)
        expected = (
            'ajax:target="http://example.com/'
            '?navigable=&referencable=&root=/'
            '&selected=745dd2ed-74ad-449b-b55b-b240709b2b0e"'
        )
        self.assertTrue(widget(data=data).find(expected) > -1)

        request = self.layer.new_request()

        widget = factory('reference', name='ref', props={'multivalued': True})
        data = widget.extract(request)
        expected = (
            'ajax:target="http://example.com/'
            '?navigable=&referencable=&root=/&selected="'
        )
        self.assertTrue(widget(data=data).find(expected) > -1)
        del widget.attrs['vocabulary']

        widget.getter = ['5f94f75c-4aea-4dc7-a31b-6152e8f89d00']
        data = widget.extract(request)
        expected = (
            'ajax:target="http://example.com/'
            '?navigable=&referencable=&root=/'
            '&selected=5f94f75c-4aea-4dc7-a31b-6152e8f89d00"'
        )
        self.assertTrue(widget(data=data).find(expected) > -1)
        del widget.attrs['vocabulary']

        request.params['ref-exists'] = 'exists'
        request.params['ref'] = ['e2121521-60a9-4fe6-babf-0f86881c3d6a']
        data = widget.extract(request)
        expected = (
            'ajax:target="http://example.com/'
            '?navigable=&referencable=&root=/'
            '&selected=e2121521-60a9-4fe6-babf-0f86881c3d6a"'
        )
        self.assertTrue(widget(data=data).find(expected) > -1)

    def test_multivalued_reference_vocab(self):
        self.layer.new_request()
        widget = factory(
            'reference',
            name='ref',
            props={
                'multivalued': True
            })

        # when widget gets called on multivalued reference, vocabulaty gets
        # set on widget props for proper functioning of selection
        # renderer/extractor
        widget()
        self.assertTrue('vocabulary' in widget.attrs)
        self.assertEqual(widget.attrs['vocabulary'], [])
        del widget.attrs['vocabulary']

        # case no preset value and value passed on request.
        # no lookup function given, label is uuid
        self.assertEqual(widget.getter, UNSET)
        request = self.layer.new_request()
        request.params['ref'] = 'f5c4f643-1bbd-481e-a8b0-8a47ca070184'
        widget(request=request)
        self.assertEqual(widget.attrs['vocabulary'], [(
            'f5c4f643-1bbd-481e-a8b0-8a47ca070184',
            'f5c4f643-1bbd-481e-a8b0-8a47ca070184'
        )])
        del widget.attrs['vocabulary']

        # case preset value and no value passed on request.
        # no lookup function given, label is uuid
        request = self.layer.new_request()
        widget.getter = ['94790e41-1441-44b3-b196-7c4d73bb9cca']
        widget(request=request)
        self.assertEqual(widget.attrs['vocabulary'], [(
            '94790e41-1441-44b3-b196-7c4d73bb9cca',
            '94790e41-1441-44b3-b196-7c4d73bb9cca'
        )])
        del widget.attrs['vocabulary']

        # case preset value and value passed on request.
        # request value takes precedence
        # no lookup function given, label is uuid
        request = self.layer.new_request()
        request.params['ref'] = ['2faba3ab-5af0-4240-b457-8c65ac87b8fa']
        widget.getter = ['180eb583-239f-48dc-b304-6302296939a5']
        widget(request=request)
        self.assertEqual(widget.attrs['vocabulary'], [(
            '2faba3ab-5af0-4240-b457-8c65ac87b8fa',
            '2faba3ab-5af0-4240-b457-8c65ac87b8fa'
        )])
        del widget.attrs['vocabulary']

        # case preset value and empty value passed on request.
        # request value takes precedence
        # no lookup function given, label is uuid
        request = self.layer.new_request()
        request.params['ref'] = []
        widget.getter = ['286abf9d-f690-4c34-bfc3-600722e955d3']
        widget(request=request)
        self.assertEqual(widget.attrs['vocabulary'], [])
        del widget.attrs['vocabulary']

        # case preset value is a callable.
        def value_getter(widget, data):
            return ['d9908598-e592-45df-9e6a-f88512dddf29']

        request = self.layer.new_request()
        widget.getter = value_getter
        widget(request=request)
        self.assertEqual(widget.attrs['vocabulary'], [(
            'd9908598-e592-45df-9e6a-f88512dddf29',
            'd9908598-e592-45df-9e6a-f88512dddf29'
        )])
        del widget.attrs['vocabulary']

        # dummy lookup function
        def label_lookup(uuid_):
            return {
                '8208af4f-522f-436e-93b3-37e4e736984d': 'Item 1',
                'bb6a21f9-fb35-4d76-9063-1b0007a9df52': 'Item 2'
            }.get(uuid_, uuid_)

        # case lookup function, preset value and no value passed on request
        request = self.layer.new_request()
        widget.attrs['lookup'] = label_lookup
        widget.getter = [
            '8208af4f-522f-436e-93b3-37e4e736984d',
            'bb6a21f9-fb35-4d76-9063-1b0007a9df52'
        ]
        widget(request=request)
        self.assertEqual(widget.attrs['vocabulary'], [
            ('8208af4f-522f-436e-93b3-37e4e736984d', 'Item 1'),
            ('bb6a21f9-fb35-4d76-9063-1b0007a9df52', 'Item 2')
        ])
        del widget.attrs['vocabulary']

        # case lookup function, preset value and empty value passed on request
        request = self.layer.new_request()
        request.params['ref'] = []
        widget(request=request)
        self.assertEqual(widget.attrs['vocabulary'], [])
        del widget.attrs['vocabulary']

        # case lookup function, preset value and value passed on request
        request = self.layer.new_request()
        request.params['ref'] = ['8208af4f-522f-436e-93b3-37e4e736984d']
        widget(request=request)
        self.assertEqual(widget.attrs['vocabulary'], [
            ('8208af4f-522f-436e-93b3-37e4e736984d', 'Item 1')
        ])
        del widget.attrs['vocabulary']

        # case lookup function, preset value and unknown value passed on request
        request = self.layer.new_request()
        request.params['ref'] = ['c1cfaa74-48e8-46e8-a16c-fba52ed8c3d4']
        widget(request=request)
        self.assertEqual(widget.attrs['vocabulary'], [(
            'c1cfaa74-48e8-46e8-a16c-fba52ed8c3d4',
            'c1cfaa74-48e8-46e8-a16c-fba52ed8c3d4'
        )])
        del widget.attrs['vocabulary']

        # case B/C vocabulary
        del widget.attrs['lookup']
        request = self.layer.new_request()
        widget.attrs['vocabulary'] = [
            ('81fc7ba9-5c68-4590-a1e7-18d4309bfb1e', 'B/C Item 1'),
            ('8b190a62-f530-4ce8-9736-ec45c5de9a59', 'B/C Item 2')
        ]
        widget(request=request)
        self.assertEqual(widget.attrs['vocabulary'], [
            ('81fc7ba9-5c68-4590-a1e7-18d4309bfb1e', 'B/C Item 1'),
            ('8b190a62-f530-4ce8-9736-ec45c5de9a59', 'B/C Item 2')
        ])
        self.assertEqual(widget.attrs['bc_vocabulary'], [
            ('81fc7ba9-5c68-4590-a1e7-18d4309bfb1e', 'B/C Item 1'),
            ('8b190a62-f530-4ce8-9736-ec45c5de9a59', 'B/C Item 2')
        ])
        del widget.attrs['vocabulary']
        del widget.attrs['bc_vocabulary']

    def test_single_reference(self):
        # no preset value
        widget = factory('reference', name='ref')

        request = self.layer.new_request()
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [UNSET, UNSET, []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<input name="ref.uid" type="hidden" value="" />'
        ) > -1)
        self.assertTrue(rendered.find(
            '<input class="form-control referencebrowser" id="input-ref" '
            'name="ref" readonly="readonly" type="text" value="" />'
        ) > -1)

        request.params['ref'] = ''
        request.params['ref.uid'] = ''
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [UNSET, '', []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<input name="ref.uid" type="hidden" value="" />'
        ) > -1)
        self.assertTrue(rendered.find(
            '<input class="form-control referencebrowser" id="input-ref" '
            'name="ref" readonly="readonly" type="text" value="" />'
        ) > -1)

        request.params['ref'] = 'Reference Item'
        request.params['ref.uid'] = '378657f5-c435-4678-886b-a3eefb3141b5'
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [UNSET, '378657f5-c435-4678-886b-a3eefb3141b5', []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<input name="ref.uid" type="hidden" '
            'value="378657f5-c435-4678-886b-a3eefb3141b5" />'
        ) > -1)
        self.assertTrue(rendered.find(
            '<input class="form-control referencebrowser" id="input-ref" '
            'name="ref" readonly="readonly" type="text" value="Reference Item" />'
        ) > -1)

        # preset value
        value = ['482ba322-169b-40ef-b632-6e92f29fe0ba', 'Reference Item']
        widget = factory('reference', name='ref', value=value)

        request = self.layer.new_request()
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [value, UNSET, []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<input name="ref.uid" type="hidden" '
            'value="482ba322-169b-40ef-b632-6e92f29fe0ba" />'
        ) > -1)
        self.assertTrue(rendered.find(
            '<input class="form-control referencebrowser" id="input-ref" '
            'name="ref" readonly="readonly" type="text" value="Reference Item" />'
        ) > -1)

        request.params['ref'] = ''
        request.params['ref.uid'] = ''
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [value, '', []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<input name="ref.uid" type="hidden" value="" />'
        ) > -1)
        self.assertTrue(rendered.find(
            '<input class="form-control referencebrowser" id="input-ref" '
            'name="ref" readonly="readonly" type="text" value="" />'
        ) > -1)

        request.params['ref'] = 'Other Item'
        request.params['ref.uid'] = '81fc0e53-5551-41d4-a1bd-6064e34face5'
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [value, '81fc0e53-5551-41d4-a1bd-6064e34face5', []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<input name="ref.uid" type="hidden" '
            'value="81fc0e53-5551-41d4-a1bd-6064e34face5" />'
        ) > -1)
        self.assertTrue(rendered.find(
            '<input class="form-control referencebrowser" id="input-ref" '
            'name="ref" readonly="readonly" type="text" value="Other Item" />'
        ) > -1)

        # required value
        widget = factory('reference', name='ref', props={'required': True})

        request = self.layer.new_request()
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [UNSET, UNSET, []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<input name="ref.uid" type="hidden" value="" />'
        ) > -1)
        self.assertTrue(rendered.find(
            '<input class="form-control referencebrowser required" '
            'id="input-ref" name="ref" readonly="readonly" type="text" '
            'value="" />'
        ) > -1)

        request = self.layer.new_request()
        request.params['ref'] = ''
        request.params['ref.uid'] = ''
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [UNSET, '', [ExtractionError('Mandatory field was empty')]]
        )

        request.params['ref'] = 'Item'
        request.params['ref.uid'] = '5a65bc60-3420-43ce-bf99-9190e2ea4c02'
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [UNSET, '5a65bc60-3420-43ce-bf99-9190e2ea4c02', []]
        )

    def test_multi_reference(self):
        # no preset value
        widget = factory('reference', name='ref', props={'multivalued': True})

        request = self.layer.new_request()
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [UNSET, UNSET, []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<input id="exists-ref" name="ref-exists" '
            'type="hidden" value="exists" />'
        ) > -1)
        self.assertTrue(rendered.find(
            '<select class="form-control referencebrowser" id="input-ref" '
            'multiple="multiple" name="ref"> </select>'
        ) > -1)
        del widget.attrs['vocabulary']

        request.params['ref-exists'] = 'exists'
        request.params['ref'] = []
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [UNSET, [], []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<select class="form-control referencebrowser" id="input-ref" '
            'multiple="multiple" name="ref"> </select>'
        ) > -1)
        del widget.attrs['vocabulary']

        request.params['ref-exists'] = 'exists'
        request.params['ref'] = ['292cd228-4095-4381-9ac3-2a8680ac4669']
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [UNSET, ['292cd228-4095-4381-9ac3-2a8680ac4669'], []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<option id="input-ref-292cd228-4095-4381-9ac3-2a8680ac4669" '
            'selected="selected" value="292cd228-4095-4381-9ac3-2a8680ac4669">'
            '292cd228-4095-4381-9ac3-2a8680ac4669</option>'
        ) > -1)

        # preset value
        value = ['0bed1aa2-103d-405c-84bc-039152e4738e']
        widget = factory(
            'reference',
            name='ref',
            value=value,
            props={
                'multivalued': True
            })

        request = self.layer.new_request()
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [value, UNSET, []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<option id="input-ref-0bed1aa2-103d-405c-84bc-039152e4738e" '
            'selected="selected" value="0bed1aa2-103d-405c-84bc-039152e4738e">'
            '0bed1aa2-103d-405c-84bc-039152e4738e</option>'
        ) > -1)
        del widget.attrs['vocabulary']

        request.params['ref-exists'] = 'exists'
        request.params['ref'] = []
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [value, [], []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<select class="form-control referencebrowser" id="input-ref" '
            'multiple="multiple" name="ref"> </select>'
        ) > -1)
        del widget.attrs['vocabulary']

        request.params['ref-exists'] = 'exists'
        request.params['ref'] = ['cfaed2ab-f39b-4c70-bd56-b6d9f4f0d849']
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [value, ['cfaed2ab-f39b-4c70-bd56-b6d9f4f0d849'], []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<option id="input-ref-cfaed2ab-f39b-4c70-bd56-b6d9f4f0d849" '
            'selected="selected" value="cfaed2ab-f39b-4c70-bd56-b6d9f4f0d849">'
            'cfaed2ab-f39b-4c70-bd56-b6d9f4f0d849</option>'
        ) > -1)

        # required value
        widget = factory(
            'reference',
            name='ref',
            props={
                'required': True,
                'multivalued': True
            })

        request = self.layer.new_request()
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [UNSET, UNSET, []]
        )
        rendered = widget(data=data)
        self.assertTrue(rendered.find(
            '<select class="form-control referencebrowser required" '
            'id="input-ref" multiple="multiple" name="ref" '
            'required="required"> </select>'
        ) > -1)
        del widget.attrs['vocabulary']

        request = self.layer.new_request()
        request.params['ref-exists'] = 'exists'
        request.params['ref'] = []
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [UNSET, [], [ExtractionError('Mandatory field was empty')]]
        )

        request.params['ref-exists'] = 'exists'
        request.params['ref'] = ['8944998c-b80c-4232-a70d-7d8b426961f8']
        data = widget.extract(request)
        self.assertEqual(
            [data.value, data.extracted, data.errors],
            [UNSET, ['8944998c-b80c-4232-a70d-7d8b426961f8'], []]
        )

    def test_display_renderer(self):
        self.layer.new_request()

        # Single value display renderer
        widget = factory(
            'reference',
            name='ref',
            mode='display')
        self.assertEqual(widget(), (
            '<div class="display-referencebrowser form-control" '
            'id="display-ref"></div>'
        ))

        widget = factory(
            'reference',
            name='ref',
            value=('00c753e6-2284-4118-a286-b43d743f1259', 'Label'),
            mode='display')
        self.assertEqual(widget(), (
            '<div class="display-referencebrowser form-control" '
            'id="display-ref">Label</div>'
        ))

        # Multi value display renderer
        def lookup_label(uuid):
            return {
                '1900f873-8442-4046-9f08-c94a0a0cdf51': 'Item 1',
                '2a1aea50-f48f-479a-abe9-4b0a962f8243': 'Item 2'
            }[uuid]

        widget = factory(
            'reference',
            name='ref',
            value=[
                '1900f873-8442-4046-9f08-c94a0a0cdf51',
                '2a1aea50-f48f-479a-abe9-4b0a962f8243'
            ],
            props={
                'multivalued': True,
                'lookup': lookup_label
            },
            mode='display')
        self.assertEqual(widget(), (
            '<ul class="display-referencebrowser form-control" '
            'id="display-ref"><li>Item 1</li><li>Item 2</li></ul>'
        ))

    def test_ReferenceBrowserModelMixin(self):
        model = BaseNode()
        a = model['a'] = BaseNode()
        aa = model['a']['a'] = BaseNode()
        b = model['b'] = BaseNode()

        ref_model_mixin = ReferenceBrowserModelMixin()
        ref_model_mixin.model = model

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = '/'
        self.assertEqual(ref_model_mixin.referencable_root, model)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = ''
        self.assertEqual(ref_model_mixin.referencable_root, model)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = '/a'
        self.assertEqual(ref_model_mixin.referencable_root, a)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = '/a/a'
        self.assertEqual(ref_model_mixin.referencable_root, aa)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = 'b'
        self.assertEqual(ref_model_mixin.referencable_root, b)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = 'c'
        self.expectError(
            KeyError,
            lambda: ref_model_mixin.referencable_root
        )

        ref_model_mixin.model = aa

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = 'a/a'
        self.assertEqual(ref_model_mixin.referencebrowser_model, aa)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = 'a'
        self.assertEqual(ref_model_mixin.referencebrowser_model, aa)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = ''
        self.assertEqual(ref_model_mixin.referencebrowser_model, aa)

        ref_model_mixin.model = b

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = ''
        self.assertEqual(ref_model_mixin.referencebrowser_model, b)

        request = ref_model_mixin.request = self.layer.new_request()
        request.params['root'] = 'a'
        self.assertEqual(ref_model_mixin.referencebrowser_model, a)

    def test_ReferenceAction(self):
        model = RefNode()
        request = self.layer.new_request()

        action = ReferenceAction()
        action.model = model
        action.request = request

        request.params['selected'] = ''
        self.assertEqual(action.selected_uids, [])
        request.params['selected'] = 'foo'
        self.assertEqual(action.selected_uids, ['foo'])
        request.params['selected'] = 'foo,bar'
        self.assertEqual(action.selected_uids, ['foo', 'bar'])

        request.params['referencable'] = ''
        self.assertTrue(action.display)
        request.params['referencable'] = 'ref_node,ref_node_2'
        self.assertTrue(action.display)
        request.params['referencable'] = 'ref_node'
        self.assertTrue(action.display)
        request.params['referencable'] = 'ref_node_2'
        self.assertFalse(action.display)

        self.checkOutput('ref-...', action.id)
        self.checkOutput("""
        ...<a
        id="ref-..."
        href="http://example.com/"
        data-toggle="tooltip"
        data-placement="top"
        ajax:bind="click"
        ></a>...
        """, action.render())

        model = BaseNode()
        model.node_info_name = 'nouuid'
        request.params['referencable'] = 'nouuid'

        action.model = model
        self.assertFalse(action.display)

    def test_ActionAddReference(self):
        model = BaseNode()
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['selected'] = ''
        request.params['root'] = '/'

        action = ActionAddReference()
        self.assertEqual(action(model, request), u'')

        with self.layer.authenticated('manager'):
            self.assertEqual(action(model, request), u'')

        model = RefNode(name='model')
        rendered = action(model, request)
        expected = 'class="addreference"'
        self.assertTrue(rendered.find(expected) > -1)
        expected = 'title="Add reference"'
        self.assertTrue(rendered.find(expected) > -1)
        expected = '<span class="glyphicons glyphicons-plus-sign">'
        self.assertTrue(rendered.find(expected) > -1)
        expected = '<span class="reftitle" style="display:none;">model</span>'
        self.assertTrue(rendered.find(expected) > -1)

        request.params['selected'] = str(model.uuid)
        rendered = action(model, request)
        expected = 'class="addreference disabled"'
        self.assertTrue(rendered.find(expected) > -1)

        action = ActionAddReference()
        action.model = BaseNode(name='model')
        action.request = self.layer.new_request()
        self.assertFalse(action.enabled)

    def test_ActionRemoveReference(self):
        model = BaseNode()
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['selected'] = ''
        request.params['root'] = '/'

        action = ActionRemoveReference()
        self.assertEqual(action(model, request), u'')

        with self.layer.authenticated('manager'):
            self.assertEqual(action(model, request), u'')

        model = RefNode(name='model')
        rendered = action(model, request)
        expected = 'class="removereference disabled"'
        self.assertTrue(rendered.find(expected) > -1)
        expected = 'title="Remove reference"'
        self.assertTrue(rendered.find(expected) > -1)
        expected = '<span class="glyphicons glyphicons-minus-sign">'
        self.assertTrue(rendered.find(expected) > -1)
        expected = '<span class="reftitle" style="display:none;">model</span>'
        self.assertTrue(rendered.find(expected) > -1)

        request.params['selected'] = str(model.uuid)
        rendered = action(model, request)
        expected = 'class="removereference"'
        self.assertTrue(rendered.find(expected) > -1)

        action = ActionRemoveReference()
        action.model = BaseNode(name='model')
        action.request = self.layer.new_request()
        self.assertFalse(action.enabled)

    def test_ReferencableChildrenLink(self):
        model = BaseNode(name='model')
        model.metadata.title = 'My Node'
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['navigable'] = ''
        request.params['selected'] = ''
        request.params['root'] = '/'

        table_tile_name = 'tabletile'
        table_id = 'tableid'

        action = ReferencableChildrenLink(table_tile_name, table_id)
        action.model = model
        action.request = request

        self.assertTrue(isinstance(action, LinkAction))

        expected = (
            'http://example.com/model'
            '?navigable=&referencable=ref_node&root=/&selected='
        )
        self.assertEqual(action.target, expected)

        expected = 'My Node'
        self.assertEqual(action.text, expected)

        expected = 'tabletile:#tableid:replace'
        self.assertEqual(action.action, expected)

        expected = 'contextchanged:.refbrowsersensitiv'
        self.assertEqual(action.event, expected)

        self.assertFalse(action.display)
        with self.layer.authenticated('max'):
            self.assertTrue(action.display)

        expected = 'glyphicon glyphicon-asterisk'
        self.assertEqual(action.icon, expected)

        self.assertEqual(action(model, request), u'')

        with self.layer.authenticated('manager'):
            rendered = action(model, request)
        expected = '<a'
        self.assertTrue(rendered.find(expected) > -1)
        expected = '&nbsp;My Node</a>'
        self.assertTrue(rendered.find(expected) > -1)

        @implementer(INavigationLeaf)
        class LeafNode(BaseNode):
            pass

        model = LeafNode(name='leaf')
        with self.layer.authenticated('manager'):
            rendered = action(model, request)
        expected = (
            '<span class="glyphicon glyphicon-asterisk" />'
            '&nbsp;<span>leaf</span>')
        self.assertEqual(rendered, expected)

    def test_reference_pathbar(self):
        model = BaseNode()
        model['a'] = RefNode()
        model['a']['b'] = RefNode()
        model['z'] = RefNode()
        node = model['a']['b']['c'] = RefNode()

        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['navigable'] = ''
        request.params['selected'] = ''
        request.params['root'] = '/'

        # Case Unauthorized
        err = self.expectError(
            HTTPForbidden,
            render_tile,
            node,
            request,
            'referencebrowser_pathbar'
        )
        self.checkOutput("""
        Unauthorized: tile
        <cone.app.browser.referencebrowser.ReferenceBrowserPathBar object at ...>
        failed permission check
        """, str(err))

        # Case reference root is application root
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['navigable'] = ''
        request.params['selected'] = ''
        request.params['root'] = '/'
        with self.layer.authenticated('max'):
            res = render_tile(node, request, 'referencebrowser_pathbar')
        self.assertTrue(res.find('"http://example.com/?') > -1)
        self.assertTrue(res.find('"http://example.com/a?') > -1)
        self.assertTrue(res.find('"http://example.com/a/b?') > -1)

        # Case reference root is in current sub tree
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['navigable'] = ''
        request.params['selected'] = ''
        request.params['root'] = 'a'
        with self.layer.authenticated('max'):
            res = render_tile(node, request, 'referencebrowser_pathbar')
        self.assertFalse(res.find('"http://example.com/?') > -1)
        self.assertTrue(res.find('"http://example.com/a?') > -1)
        self.assertTrue(res.find('"http://example.com/a/b?') > -1)

        # Case reference root is in sibling sub tree
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['navigable'] = ''
        request.params['selected'] = ''
        request.params['root'] = '/z'
        with self.layer.authenticated('max'):
            res = render_tile(node, request, 'referencebrowser_pathbar')
        self.assertFalse(res.find('"http://example.com/?') > -1)
        self.assertFalse(res.find('"http://example.com/a?') > -1)
        self.assertFalse(res.find('"http://example.com/a/b?') > -1)
        self.assertTrue(res.find('<strong>z</strong>') > -1)

    def test_reference_listing(self):
        created = datetime(2011, 3, 15)
        delta = timedelta(1)
        modified = created + delta

        model = BaseNode()
        for i in range(20):
            model[str(i)] = RefNode()
            # set listing display metadata
            model[str(i)].metadata.title = str(i)
            model[str(i)].metadata.created = created
            model[str(i)].metadata.modified = modified
            if i % 2 == 0:
                # make node referencable
                model[str(i)].properties.action_add_reference = True
                # do not render link to children
                model[str(i)].properties.leaf = True
            created = created + delta
            modified = modified + delta

        # Unauthorized fails
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['navigable'] = ''
        request.params['selected'] = ''
        request.params['root'] = '/'

        err = self.expectError(
            HTTPForbidden,
            render_tile,
            model,
            request,
            'referencelisting'
        )
        self.checkOutput("""
            Unauthorized: tile
            <cone.app.browser.referencebrowser.ReferenceListing object at ...>
            failed permission check
        """, str(err))

        # Authorized
        with self.layer.authenticated('max'):
            res = render_tile(model, request, 'referencelisting')
        self.assertTrue(res.find('id="referencebrowser"') > -1)
        self.checkOutput('...<div id="referencebrowser"...', res)

        # Referencable nodes renders add reference action related markup
        self.checkOutput("""
        ...
        <a
        id="ref-..."
        href="http://example.com/1"
        class="addreference"
        title="Add reference"
        data-toggle="tooltip"
        data-placement="top"
        ajax:bind="click"
        ><span class="glyphicons glyphicons-plus-sign"></span></a>...
        """, res)

        # Non navigable nodes are not rendered
        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['navigable'] = 'other'
        request.params['selected'] = ''
        request.params['root'] = '/'
        reference_listing = ReferenceListing()
        reference_listing.model = model
        reference_listing.request = request
        with self.layer.authenticated('max'):
            self.assertEqual(
                reference_listing.sorted_children('created', 'desc'),
                []
            )
        with self.layer.authenticated('max'):
            request.params['navigable'] = ''
            self.assertEqual(
                reference_listing.sorted_children('created', 'desc'),
                model.values()
            )

    def test_referencebrowser(self):
        model = BaseNode()
        model['a'] = RefNode()

        request = self.layer.new_request()
        request.params['referencable'] = 'ref_node'
        request.params['navigable'] = ''
        request.params['selected'] = ''
        request.params['root'] = '/'

        # Case unauthorized
        err = self.expectError(
            HTTPForbidden,
            render_tile,
            model,
            request,
            'referencebrowser'
        )
        self.checkOutput("""
            Unauthorized: tile
            <cone.app.browser.referencebrowser.ReferenceBrowser object at ...>
            failed permission check
        """, str(err))

        # Case authorized
        with self.layer.authenticated('max'):
            res = render_tile(model, request, 'referencebrowser')
        self.assertTrue(res.find('<div class="referencebrowser">') > -1)
        self.assertTrue(res.find('<div id="referencebrowser_pathbar"') > -1)
        self.assertTrue(res.find('<div id="referencebrowser"') > -1)



# -*- coding: utf-8 -*-
from cone.app import testing
from cone.app.model import BaseNode
from cone.app.model import Properties
from cone.app.utils import add_creation_metadata
from cone.app.utils import app_config
from cone.app.utils import DatetimeHelper
from cone.app.utils import node_path
from cone.app.utils import safe_decode
from cone.app.utils import safe_encode
from cone.app.utils import timestamp
from cone.app.utils import update_creation_metadata
from datetime import datetime
from node.tests import NodeTestCase


class TestUtils(NodeTestCase):
    layer = testing.security

    def test_app_config(self):
        # Fetch application configuration
        cfg = app_config()
        self.assertTrue(isinstance(cfg, Properties))

    def test_safe_encode(self):
        self.assertEqual(safe_encode(u'äöü'), b'\xc3\xa4\xc3\xb6\xc3\xbc')
        self.assertEqual(safe_encode(b'already_string'), b'already_string')

    def test_safe_decode(self):
        self.assertEqual(safe_decode(b'\xc3\xa4\xc3\xb6\xc3\xbc'), u'äöü')
        self.assertEqual(safe_decode(u'already_unicode'), u'already_unicode')

    def test_node_path(self):
        root = BaseNode()
        root['child'] = BaseNode()
        self.assertEqual(node_path(root['child']), [u'child'])

    def test_DatetimeHelper(self):
        # Helper object for read/write operations with datetime values
        helper = DatetimeHelper()

        dt = datetime(2010, 1, 1, 10, 15)
        self.assertEqual(dt.isoformat(), '2010-01-01T10:15:00')
        self.assertEqual(helper.dt_to_iso(dt), '2010-01-01T10:15:00')

        dt = datetime(2010, 1, 1, 10, 15, 10, 5)
        self.assertEqual(dt.isoformat(), '2010-01-01T10:15:10.000005')
        self.assertEqual(helper.dt_to_iso(dt), '2010-01-01T10:15:10')

        self.assertEqual(
            helper.dt_from_iso('2010-01-01T10:15:00'),
            datetime(2010, 1, 1, 10, 15)
        )
        self.assertEqual(
            helper.dt_from_iso(u'2010-01-01T10:15:00'),
            datetime(2010, 1, 1, 10, 15)
        )

        self.assertEqual(helper.r_value(u'äöü'), u'äöü')
        self.assertEqual(helper.r_value(b'\xc3\xa4\xc3\xb6\xc3\xbc'), u'äöü')
        self.assertEqual(
            helper.r_value('2010-01-01T10:15:00'),
            datetime(2010, 1, 1, 10, 15)
        )

        self.assertEqual(helper.w_value(b'abc'), u'abc')
        self.assertEqual(helper.w_value(u'abc'), u'abc')
        self.assertEqual(helper.w_value(u'äöü'), u'äöü')
        self.assertEqual(helper.w_value(dt), u'2010-01-01T10:15:10')
        self.assertEqual(helper.w_value(0), u'0')
        self.assertEqual(helper.w_value(0.0), u'0.0')
        self.assertEqual(helper.w_value(None), u'None')
        self.assertEqual(helper.w_value(True), u'True')
        self.assertEqual(helper.w_value(False), u'False')

    def test_timestamp(self):
        self.check_output("""
        datetime.datetime(..., ..., ..., ..., ..., ..., ...)
        """, repr(timestamp()))

    def test_creation_metadata(self):
        # Creation metadata
        node = BaseNode()
        with self.layer.authenticated('editor'):
            self.assertFalse('creator' in node.attrs)
            self.assertFalse('created' in node.attrs)
            self.assertFalse('modified' in node.attrs)

            add_creation_metadata(self.layer.new_request(), node.attrs)

            self.assertTrue('creator' in node.attrs)
            self.assertTrue('created' in node.attrs)
            self.assertTrue('modified' in node.attrs)

            self.assertEqual(node.attrs['creator'], 'editor')
            created = node.attrs['created']
            modified = node.attrs['modified']

            self.assertTrue(isinstance(created, datetime))
            self.assertTrue(isinstance(modified, datetime))
            self.assertTrue(created == modified)

            update_creation_metadata(self.layer.new_request(), node.attrs)

            self.assertTrue(isinstance(node.attrs['created'], datetime))
            self.assertTrue(isinstance(node.attrs['modified'], datetime))
            self.assertTrue(created == node.attrs['created'])
            self.assertFalse(created == node.attrs['modified'])



from cone.app import cfg
from node.utils import UNSET
from yafowil.base import factory
from yafowil.base import Widget
from yafowil.compound import compound_extractor
from yafowil.compound import compound_renderer
from yafowil.utils import managedprops


@managedprops('factory')
def translation_extractor(widget, data):
    widget()
    compound_extractor(widget, data)
    translation_factory = widget.attrs.get('factory', dict)
    extracted = translation_factory()
    errors = list()
    for lang in cfg.available_languages:
        lang_data = data[lang]
        value = lang_data.extracted
        if value is UNSET:
            extracted = UNSET
            break
        extracted[lang] = lang_data.extracted
        errors += lang_data.errors
    if errors:
        # Raise first occured error. If we encounter a case where we need
        # to display multiple error messages, we need to hook them up to
        # data.errors directly here
        raise errors[0]
    return extracted


def filtered_chain(chain, keep):
    return [cb for cb in chain if cb[0] == keep]


def duplicate_widget(widget, keep):
    return Widget(
        widget.blueprints,
        filtered_chain(widget.extractors, keep),
        filtered_chain(widget.edit_renderers, keep),
        filtered_chain(widget.display_renderers, keep),
        filtered_chain(widget.preprocessors, keep),
        properties=widget.properties,
        custom=widget.custom,
        defaults=widget.defaults,
        mode=widget.mode
    )


def translation_tabs_renderer(widget, data):
    li = list()
    for idx, lang in enumerate(cfg.available_languages):
        lang_data = data.get(lang)
        has_errors = lang_data and lang_data.has_errors
        lang_text = lang.upper()
        if has_errors:
            lang_text = u'* {}'.format(lang_text)
        a = data.tag('a', lang_text, href=u'#translation-{}-{}'.format(
            widget.dottedpath.replace(u'.', u'-'),
            lang
        ))
        li_css = ['active'] if idx == 0 else []
        if has_errors:
            li_css.append('error')
        li_css = ' '.join(li_css) if li_css else None
        li.append(data.tag('li', a, class_=li_css))
    return data.tag('ul', *li, class_='nav nav-pills translation-nav')


factory.register(
    'translationtabs',
    edit_renderers=[
        translation_tabs_renderer
    ],
    display_renderers=[
        translation_tabs_renderer
    ]
)


def translation_edit_renderer(widget, data):
    widget['tabs'] = factory(
        'translationtabs',
        props={
            'structural': True,
        }
    )
    translations = widget['translations'] = factory(
        'div',
        props={
            'class': 'translation-fields',
            'structural': True
        })
    for lang in cfg.available_languages:
        translation = translations['translation_{}'.format(lang)] = factory(
            'div',
            props={
                'id': 'translation-{}-{}'.format(
                    widget.dottedpath.replace(u'.', u'-'),
                    lang
                ),
                'structural': True
            })
        lang_widget = duplicate_widget(widget, widget.blueprints[-1])
        # prevent persiting on generated child widgets
        lang_widget.attrs['persist'] = False
        translation[lang] = lang_widget


def translation_display_renderer(widget, data):
    translation_edit_renderer(widget, data)
    translations = widget['translations']
    for lang in cfg.available_languages:
        translation = translations['translation_{}'.format(lang)]
        translation[lang].mode = 'display'


factory.register(
    'translation',
    extractors=[
        translation_extractor
    ],
    edit_renderers=[
        translation_edit_renderer,
        compound_renderer
    ],
    display_renderers=[
        translation_display_renderer,
        compound_renderer
    ]
)

factory.doc['blueprint']['translation'] = """\
Widget for rendering translations.
"""

factory.defaults['translation.factory'] = None
factory.doc['props']['translation.factory'] = """\
A class used as factory for creating translations at extraction time.
"""



from cone.app import testing
from cone.app.browser.actions import Action
from cone.app.browser.actions import ActionContext
from cone.app.browser.actions import LinkAction
from cone.app.browser.contextmenu import context_menu
from cone.app.browser.contextmenu import context_menu_group
from cone.app.browser.contextmenu import context_menu_item
from cone.app.browser.contextmenu import ContextMenuDropdown
from cone.app.browser.contextmenu import ContextMenuToolbar
from cone.app.model import BaseNode
from cone.app.testing.mock import CopySupportNode
from cone.app.testing.mock import SharingNode
from cone.tile import render_tile
from cone.tile.tests import TileTestCase


class TestBrowserContextmenu(TileTestCase):
    layer = testing.security

    def test_ContextMenuToolbar(self):
        model = BaseNode()
        request = self.layer.new_request()

        cmt = ContextMenuToolbar()
        cmt.display = False
        self.assertEqual(cmt(model, request), '')

        class EmptyRenderAction(Action):
            def render(self):
                return ''

        cmt = ContextMenuToolbar()
        cmt['empty'] = EmptyRenderAction()
        self.assertEqual(cmt(model, request), '')

        class RenderAction(Action):
            def render(self):
                return '<span>Rendered action</span>'

        cmt = ContextMenuToolbar()
        cmt['action'] = RenderAction()
        self.assertEqual(
            cmt(model, request),
            '<ul class="nav navbar-nav"><span>Rendered action</span></ul>'
        )

        class MyLinkAction(LinkAction):
            id = 'myaction'
            icon = 'myicon'
            text = 'My Action'
            href = 'http://example.com'

        cmt = ContextMenuToolbar()
        cmt['link'] = MyLinkAction()
        self.assertTrue(cmt(model, request).find('My Action') > -1)
        self.assertFalse(cmt(model, request).find('<li class="active">') > -1)

        cmt['link'].selected = True
        self.assertTrue(cmt(model, request).find('<li class="active">') > -1)

        cmt.css = 'mytoolbar'
        expected = 'class="nav navbar-nav mytoolbar"'
        self.assertTrue(cmt(model, request).find(expected) > -1)

    def test_ContextMenuDropdown(self):
        cmd = ContextMenuDropdown(title='Dropdown')

        def add_action():
            cmd['invalid'] = Action()
        err = self.expectError(ValueError, add_action)
        expected = (
            'Only ``TemplateAction`` deriving objects can be added '
            'to ``ContextMenuDropdown`` instances.'
        )
        self.assertEqual(str(err), expected)

        class MyLinkAction(LinkAction):
            id = 'myaction'
            icon = 'myicon'
            text = 'My Action'
            href = 'http://example.com'

        cmd = ContextMenuDropdown(title='Dropdown')
        cmd.model = BaseNode()
        cmd.request = self.layer.new_request()

        cmd['link'] = MyLinkAction()
        self.assertTrue(cmd.display)

        cmd['link'].display = False
        self.assertFalse(cmd.display)

        cmd = ContextMenuDropdown(title='Dropdown')
        cmd['link'] = MyLinkAction()

        model = BaseNode()
        request = self.layer.new_request()
        res = cmd(model, request)
        self.assertTrue(res.find('<li class="dropdown">') > -1)

    def test_context_menu_decorators(self):
        @context_menu_group(name='testgroup')
        class TestContextMenuGroup(ContextMenuToolbar):
            pass

        self.assertTrue('testgroup' in context_menu)
        self.assertIsInstance(context_menu['testgroup'], TestContextMenuGroup)

        @context_menu_item(group='testgroup', name='testaction')
        class TestAction(LinkAction):
            pass

        self.assertTrue('testaction' in context_menu['testgroup'])
        self.assertIsInstance(context_menu['testgroup']['testaction'], TestAction)

        del context_menu['testgroup']

    def test_contextmenu(self):
        with self.layer.authenticated('manager'):
            parent = BaseNode(name='root')
            model = parent['model'] = SharingNode()
            model.properties.action_up = True
            model.properties.action_view = True
            model.properties.action_list = True
            model.properties.action_edit = True
            model.properties.action_delete = True
            # XXX: model.properties.action_delete_children = True

            request = self.layer.new_request()
            ActionContext(model, request, 'content')
            rendered = render_tile(model, request, 'contextmenu')

            self.assertTrue(rendered.find('toolbaraction-up') > -1)
            self.assertTrue(rendered.find('toolbaraction-view') > -1)
            self.assertTrue(rendered.find('toolbaraction-list') > -1)
            self.assertTrue(rendered.find('toolbaraction-edit') > -1)
            self.assertTrue(rendered.find('toolbaraction-delete') > -1)
            self.assertTrue(rendered.find('toolbaraction-share') > -1)

            model = CopySupportNode()
            ActionContext(model, request, 'listing')
            rendered = render_tile(model, request, 'contextmenu')

            self.assertTrue(rendered.find('toolbaraction-cut') > -1)
            self.assertTrue(rendered.find('toolbaraction-copy') > -1)
            self.assertTrue(rendered.find('toolbaraction-paste') > -1)



from cone.app.testing import DummyRequest
from cone.app.testing import Security
from cone.app.testing import security
import unittest


class TestTesting(unittest.TestCase):
    layer = security

    def test_layer(self):
        # Check layer instance
        self.assertTrue(isinstance(self.layer, Security))

        # Login with inexistent user
        with self.layer.authenticated('inexistent'):
            req = self.layer.new_request()
            self.assertTrue(isinstance(req, DummyRequest))
            self.assertTrue(req.authenticated_userid is None)
            self.assertTrue(self.layer.current_request is req)
            self.assertEqual(
                sorted(req.environ.keys()),
                ['AUTH_TYPE', 'SERVER_NAME']
            )

        # Login with existing user
        with self.layer.authenticated('max'):
            self.assertEqual(
                sorted(req.environ.keys()),
                ['AUTH_TYPE', 'HTTP_COOKIE', 'SERVER_NAME']
            )
            self.assertEqual(req.environ['AUTH_TYPE'], 'cookie')
            self.assertTrue(req.environ['HTTP_COOKIE'].startswith('auth_tkt='))
            self.assertEqual(req.environ['SERVER_NAME'], 'testcase')

            self.assertTrue(self.layer.current_request is req)
            self.assertEqual(req.authenticated_userid, 'max')
            self.assertEqual(sorted(req.environ.keys()), [
                'AUTH_TYPE', 'HTTP_COOKIE', 'REMOTE_USER_DATA',
                'REMOTE_USER_TOKENS', 'SERVER_NAME', 'cone.app.user.roles'
            ])

        # Logged out
        self.assertTrue(self.layer.current_request is req)
        self.assertEqual(
            sorted(req.environ.keys()),
            ['AUTH_TYPE', 'SERVER_NAME']
        )
        self.assertTrue(req.authenticated_userid is None)

        # Create new request and check if instance changed
        old = req
        req = self.layer.new_request()
        self.assertFalse(old is req)

        # Check auto request creation on login
        self.layer.current_request = None
        with self.layer.authenticated('max'):
            req = self.layer.current_request
            self.assertTrue(isinstance(req, DummyRequest))

        # Request accept property
        req = self.layer.new_request()
        self.assertEqual(
            req.accept.parsed,
            [('text/html', 1.0, [], [])]
        )

        del req.accept
        self.assertEqual(req.accept.parsed, None)

        # Create JSON request
        req = self.layer.new_request(type='json')
        self.assertEqual(req.headers, {'X-Request': 'JSON'})
        self.assertEqual(
            req.accept.parsed,
            [('application/json', 1.0, [], [])]
        )



from cone.app.browser.actions import ActionContext
from cone.app.browser.utils import format_traceback
from cone.app.interfaces import ILiveSearch
from cone.tile import render_tile
from cone.tile import Tile
from cone.tile import tile
from node.utils import safe_encode
from pyramid.exceptions import Forbidden
from pyramid.response import Response
from pyramid.view import view_config
import json
import logging


@tile(name='bdajax', path='bdajax:bdajax_bs3.pt', permission='login')
class BDAjaxTile(Tile):
    """Tile rendering bdajax markup.
    """


@view_config(name='ajaxaction', accept='application/json', renderer='json')
def ajax_tile(model, request):
    """bdajax ``ajaxaction`` implementation for cone.

    * Renders tile with name ``bdajax.action``.

    * Uses definitions from ``request.environ['cone.app.continuation']``
      for continuation definitions.
    """
    try:
        name = request.params['bdajax.action']
        ActionContext(model, request, name)
        rendered = render_tile(model, request, name)
        continuation = request.environ.get('cone.app.continuation')
        if continuation:
            continuation = AjaxContinue(continuation).definitions
        else:
            continuation = False
        return {
            'mode': request.params.get('bdajax.mode'),
            'selector': request.params.get('bdajax.selector'),
            'payload': rendered,
            'continuation': continuation,
        }
    except Forbidden:
        request.response.status = 403
        return {}
    except Exception:
        logging.exception('Error within ajax tile')
        tb = format_traceback()
        continuation = AjaxContinue(
            [AjaxMessage(tb, 'error', None)]).definitions
        return {
            'mode': 'NONE',
            'selector': 'NONE',
            'payload': '',
            'continuation': continuation,
        }


def ajax_continue(request, continuation):
    """Set ajax continuation on environ.

    continuation
        list of continuation definition objects or single continuation
        definition.
    """
    if request.environ.get('cone.app.continuation', None) is None:
        request.environ['cone.app.continuation'] = list()
    if isinstance(continuation, list):
        existent = request.environ['cone.app.continuation']
        request.environ['cone.app.continuation'] = existent + continuation
    else:
        request.environ['cone.app.continuation'].append(continuation)


def ajax_message(request, payload, flavor='message'):
    """Convenience to add ajax message definition to ajax continuation
    definitions.
    """
    ajax_continue(request, AjaxMessage(payload, flavor, None))


def ajax_status_message(request, payload):
    """Convenience to add ajax status message definition to ajax continuation
    definitions.
    """
    ajax_continue(request, AjaxMessage(payload, None, '#status_message'))


class AjaxPath(object):
    """Ajax path configuration. Used to define continuation path for
    client side.
    """

    def __init__(self, path, target=None,
                 action=None, event=None,
                 overlay=None, overlay_css=None):
        self.path = path
        self.target = target
        self.action = action
        self.event = event
        self.overlay = overlay
        self.overlay_css = overlay_css


class AjaxAction(object):
    """Ajax action configuration. Used to define continuation actions for
    client side.
    """

    def __init__(self, target, name, mode, selector):
        self.target = target
        self.name = name
        self.mode = mode
        self.selector = selector


class AjaxEvent(object):
    """Ajax event configuration. Used to define continuation events for
    client side.
    """

    def __init__(self, target, name, selector, data=None):
        self.target = target
        self.name = name
        self.selector = selector
        self.data = data


class AjaxMessage(object):
    """Ajax Message configuration. Used to define continuation messages for
    client side.
    """

    def __init__(self, payload, flavor, selector):
        self.payload = payload
        self.flavor = flavor
        self.selector = selector


class AjaxOverlay(object):
    """Ajax overlay configuration. Used to display or close overlays on client
    side.
    """

    def __init__(self, selector='#ajax-overlay', action=None, target=None,
                 close=False, content_selector='.overlay_content', css=None):
        self.selector = selector
        self.content_selector = content_selector
        self.css = css
        self.action = action
        self.target = target
        self.close = close


class AjaxContinue(object):
    """Convert ``AjaxPath``, ``AjaxAction``, ``AjaxEvent``, ``AjaxMessage``
    and  ``AjaxOverlay ``instances to JSON response definitions for bdajax
    continuation.
    """

    def __init__(self, continuation):
        self.continuation = continuation

    @property
    def definitions(self):
        """Continuation definitions as list of dicts for JSON serialization.
        """
        if not self.continuation:
            return
        continuation = list()
        for definition in self.continuation:
            if isinstance(definition, AjaxPath):
                continuation.append({
                    'type': 'path',
                    'path': definition.path,
                    'target': definition.target,
                    'action': definition.action,
                    'event': definition.event,
                    'overlay': definition.overlay,
                    'overlay_css': definition.overlay_css
                })
            if isinstance(definition, AjaxAction):
                continuation.append({
                    'type': 'action',
                    'target': definition.target,
                    'name': definition.name,
                    'mode': definition.mode,
                    'selector': definition.selector
                })
            if isinstance(definition, AjaxEvent):
                continuation.append({
                    'type': 'event',
                    'target': definition.target,
                    'name': definition.name,
                    'selector': definition.selector,
                    'data': definition.data
                })
            if isinstance(definition, AjaxMessage):
                continuation.append({
                    'type': 'message',
                    'payload': definition.payload,
                    'flavor': definition.flavor,
                    'selector': definition.selector
                })
            if isinstance(definition, AjaxOverlay):
                continuation.append({
                    'type': 'overlay',
                    'selector': definition.selector,
                    'content_selector': definition.content_selector,
                    'css': definition.css,
                    'action': definition.action,
                    'target': definition.target,
                    'close': definition.close
                })
        return continuation

    def dump(self):
        """Return a JSON dump of continuation definitions.
        """
        ret = self.definitions
        if not ret:
            return
        return json.dumps(ret)


class AjaxFormContinue(AjaxContinue):
    """Ajax form continuation computing. Used by ``render_ajax_form``.
    """

    def __init__(self, result, continuation):
        self.result = result
        AjaxContinue.__init__(self, continuation)

    @property
    def form(self):
        """Return rendered form tile result if no continuation actions.
        """
        if not self.continuation:
            return self.result
        return ''

    @property
    def next(self):
        """Return 'false' if no continuation actions, otherwise a JSON dump of
        continuation definitions.
        """
        ret = self.dump()
        if not ret:
            return 'false'
        return ret


def ajax_form_fiddle(request, selector, mode):
    """Define ajax form fiddle mode and selector. Used on client side to
    determine form location in replacement mode for rendered ajax form.
    """
    request.environ['cone.app.form.selector'] = selector
    request.environ['cone.app.form.mode'] = mode


ajax_form_template = """\
<div id="ajaxform">
    %(form)s
</div>
<script language="javascript" type="text/javascript">
    var container = document.getElementById('ajaxform');
    var child = container.firstChild;
    while(child != null && child.nodeType == 3) {
        child = child.nextSibling;
    }
    parent.bdajax.render_ajax_form(child, '%(selector)s', '%(mode)s', %(next)s);
</script>
"""


def render_ajax_form(model, request, name):
    """Render ajax form.
    """
    try:
        result = render_tile(model, request, name)
        selector = request.environ.get('cone.app.form.selector', '#content')
        mode = request.environ.get('cone.app.form.mode', 'inner')
        continuation = request.environ.get('cone.app.continuation')
        form_continue = AjaxFormContinue(result, continuation)
        rendered_form = form_continue.form
        rendered = ajax_form_template % {
            'form': rendered_form,
            'selector': selector,
            'mode': mode,
            'next': form_continue.next
        }
        request.response.body = safe_encode(rendered)
        return request.response
    except Exception:
        result = '<div>Form rendering error</div>'
        selector = request.environ.get('cone.app.form.selector', '#content')
        mode = request.environ.get('cone.app.form.mode', 'inner')
        tb = format_traceback()
        continuation = AjaxMessage(tb, 'error', None)
        form_continue = AjaxFormContinue(result, [continuation])
        rendered = ajax_form_template % {
            'form': form_continue.form.replace(u'\n', u' '),  # XXX: replace?
            'selector': selector,
            'mode': mode,
            'next': form_continue.next
        }
        return Response(rendered)


@view_config(name='livesearch', accept='application/json', renderer='json')
def livesearch(model, request):
    adapter = request.registry.queryAdapter(model, ILiveSearch)
    if not adapter:
        return list()
    return adapter.search(request, request.params['term'])



from cone.app.interfaces import IACLAdapter
from cone.app.interfaces import IAdapterACL
from cone.app.interfaces import IAuthenticator
from cone.app.interfaces import IOwnerSupport
from cone.app.interfaces import IPrincipalACL
from cone.app.ugm import ugm_backend
from plumber import Behavior
from plumber import default
from plumber import override
from plumber import plumb
from pyramid.i18n import TranslationStringFactory
from pyramid.security import ALL_PERMISSIONS
from pyramid.security import Allow
from pyramid.security import Deny
from pyramid.security import Everyone
from pyramid.security import remember
from pyramid.threadlocal import get_current_request
from zope.interface import implementer
import logging


logger = logging.getLogger('cone.app')
_ = TranslationStringFactory('cone.app')


DEFAULT_ROLES = [
    ('viewer', _('role_viewer', default='Viewer')),
    ('editor', _('role_editor', default='Editor')),
    ('admin', _('role_admin', default='Admin')),
    ('manager', _('role_manager', default='Manager')),
]


authenticated_permissions = [
    'view',
]
viewer_permissions = [
    'view', 'list',
]
editor_permissions = [
    'view', 'list', 'add', 'edit', 'change_order'
]
owner_permissions = [
    'view', 'list', 'add', 'edit', 'change_order', 'delete', 'cut',
    'copy', 'paste', 'manage_permissions', 'change_state',
]
admin_permissions = [
    'view', 'list', 'add', 'edit', 'change_order', 'delete', 'cut',
    'copy', 'paste', 'manage_permissions', 'change_state',
]
manager_permissions = [
    'view', 'list', 'add', 'edit', 'change_order', 'delete', 'cut',
    'copy', 'paste', 'manage_permissions', 'change_state', 'manage',
]
everyone_permissions = [
    'login',
]
DEFAULT_ACL = [
    (Allow, 'system.Authenticated', authenticated_permissions),
    (Allow, 'role:viewer', viewer_permissions),
    (Allow, 'role:editor', editor_permissions),
    (Allow, 'role:admin', admin_permissions),
    (Allow, 'role:manager', manager_permissions),
    (Allow, 'role:owner', owner_permissions),
    (Allow, Everyone, everyone_permissions),
    (Deny, Everyone, ALL_PERMISSIONS),
]


settings_manager_permissions = [
    'view', 'edit', 'manage',
]
DEFAULT_SETTINGS_ACL = [
    (Allow, 'role:manager', settings_manager_permissions),
    (Allow, Everyone, everyone_permissions),
    (Deny, Everyone, ALL_PERMISSIONS),
]


ADMIN_USER = None
ADMIN_PASSWORD = None
AUTHENTICATOR = None


def authenticate(request, login, password):
    if ADMIN_USER and ADMIN_PASSWORD:
        if login == ADMIN_USER and password == ADMIN_PASSWORD:
            return remember(request, login)
    if AUTHENTICATOR:
        authenticator = request.registry.queryUtility(
            IAuthenticator,
            name=AUTHENTICATOR
        )
        if authenticator:
            pid = authenticator.authenticate(login, password)
            if pid:
                return remember(request, pid)
    ugm = ugm_backend.ugm
    try:
        if ugm.users.authenticate(login, password):
            pid = ugm.users.id_for_login(login)
            return remember(request, pid)
    except Exception as e:
        logger.warning((
            u'Authentication plugin {} raised an Exception while '
            u'trying to authenticate: {}'
        ).format(str(ugm.__class__), e))


def authenticated_user(request):
    user_id = request.authenticated_userid
    return principal_by_id(user_id)


def principal_by_id(principal_id):
    ugm = ugm_backend.ugm
    try:
        if principal_id.startswith('group:'):
            principal = ugm.groups.get(principal_id[6:])
        else:
            principal = ugm.users.get(principal_id)
        if not principal:
            return None
        return principal
    except Exception:
        return None


def search_for_principals(term):
    ret = list()
    criteria = {
        'id': term,
    }
    ugm = ugm_backend.ugm
    for user in ugm.users.search(criteria=criteria, or_search=True):
        ret.append(user)
    for group in ugm.groups.search(criteria=criteria, or_search=True):
        ret.append(u'group:%s' % group)
    return ret


ROLES_CACHE_KEY = 'cone.app.user.roles'


def groups_callback(name, request):
    """Collect and return roles and groups for user.

    XXX: request caching via decorator
    """
    environ = request.environ
    roles = environ.get(ROLES_CACHE_KEY)
    if roles:
        return roles
    if name == ADMIN_USER:
        roles = environ[ROLES_CACHE_KEY] = [u'role:manager']
        return roles
    ugm = ugm_backend.ugm
    user = None
    try:
        user = ugm.users.get(name)
    except Exception as e:
        logger.error(str(e))
    roles = list()
    if user:
        aggregated = set()
        for role in user.roles:
            aggregated.add('role:%s' % role)
        for group in user.groups:
            aggregated.add('group:%s' % group.name)
            for role in group.roles:
                aggregated.add('role:%s' % role)
        roles = environ[ROLES_CACHE_KEY] = list(aggregated)
    # XXX: this function is expected to return None if no roles?
    #      owner support not works if None returned
    # if not roles:
    #     return None
    return roles


class ACLRegistry(dict):
    """ACL registry.
    """

    def register(self, acl, obj=None, node_info_name=''):
        self[(obj, node_info_name)] = acl

    def lookup(self, obj=None, node_info_name='', default=DEFAULT_ACL):
        return self.get((obj, node_info_name), default)


acl_registry = ACLRegistry()


@implementer(IOwnerSupport)
class OwnerSupport(Behavior):
    """Plumbing behavior providing ownership information.
    """
    owner_attribute_name = default('owner')

    @plumb
    def __init__(_next, self, *args, **kw):
        _next(self, *args, **kw)
        if not self.owner:
            request = get_current_request()
            self.owner = request.authenticated_userid

    @plumb
    @property
    def __acl__(_next, self):
        acl = _next(self)
        if self.owner:
            for ace in acl:
                if ace[1] == 'role:owner':
                    return [(Allow, self.owner, ace[2])] + acl
        return acl

    @property
    def owner(self):
        return self.attrs.get(self.owner_attribute_name)

    @default
    @owner.setter
    def owner(self, value):
        self.attrs[self.owner_attribute_name] = value


@implementer(IPrincipalACL)
class PrincipalACL(Behavior):
    """Plumbing behavior providing principal ACL's.

    Warning: This behavior works only for nodes defining the ``__acl__``
    attribute as property function. Plumber does not support class property
    plumbing (yet).
    """
    role_inheritance = default(False)

    @default
    @property
    def principal_roles(self):
        raise NotImplementedError(u"Abstract ``PrincipalACL`` does not "
                                  u"implement ``principal_roles``.")

    @default
    @property
    def aggregated_roles(self):
        aggregated = dict()
        model = self
        while model:
            if not IPrincipalACL.providedBy(model):
                model = model.parent
                continue
            for id, roles in model.principal_roles.items():
                if aggregated.get(id):
                    aggregated[id].update(roles)
                else:
                    aggregated[id] = set(roles)
            model = model.parent
        return aggregated

    @default
    def aggregated_roles_for(self, principal_id):
        return list(self.aggregated_roles.get(principal_id, list()))

    @plumb
    @property
    def __acl__(_next, self):
        base_acl = _next(self)
        acl = list()
        if self.role_inheritance:
            principal_roles = self.aggregated_roles
        else:
            principal_roles = self.principal_roles
        for id, roles in principal_roles.items():
            aggregated = set()
            for role in roles:
                aggregated.update(self._permissions_for_role(base_acl, role))
            acl.append((Allow, id, list(aggregated)))
        for ace in base_acl:
            acl.append(ace)
        return acl

    @default
    def _permissions_for_role(self, acl, role):
        for ace in acl:
            if ace[1] == 'role:%s' % role:
                return ace[2]
        return list()


@implementer(IAdapterACL)
class AdapterACL(Behavior):
    """Plumbing behavior providing ACL from ``IACLAdapter``
    """
    default_acl = default(DEFAULT_ACL)

    @override
    @property
    def __acl__(self):
        request = get_current_request()
        acl_adapter = request.registry.queryAdapter(
            self,
            IACLAdapter,
            default=None
        )
        if acl_adapter:
            return acl_adapter.acl
        return self.default_acl



from cone.app.browser.ajax import AjaxEvent
from cone.app.browser.utils import make_query
from cone.app.browser.utils import make_url
from cone.app.model import Properties
from cone.app.workflow import lookup_state_data
from cone.app.workflow import lookup_workflow
from cone.tile import Tile
from cone.tile import tile
from repoze.workflow import WorkflowError
import logging


logger = logging.getLogger('cone.app')


@tile(name='wf_dropdown',
      path='templates/wf_dropdown.pt',
      permission='change_state',
      strict=False)
class WfDropdown(Tile):
    """Transition dropdown.

    If ``do_transition`` is found in ``request.params``, perform given
    transition on ``self.model`` immediately before dropdown gets rendered.
    """

    def do_transition(self):
        """if ``do_transition`` is found on request.params, perform
        transition.
        """
        transition = self.request.params.get('do_transition')
        if not transition:
            return
        workflow = self.workflow
        workflow.transition(self.model, self.request, transition)
        self.model()
        url = make_url(self.request, node=self.model)
        continuation = [AjaxEvent(url, 'contextchanged', '#layout')]
        self.request.environ['cone.app.continuation'] = continuation

    @property
    def workflow(self):
        return lookup_workflow(self.model)

    @property
    def state_name(self):
        workflow_tsf = self.model.workflow_tsf
        if workflow_tsf:
            return workflow_tsf(self.model.state)
        state_data = lookup_state_data(self.model)
        return state_data.get('title', self.model.state)

    @property
    def transitions(self):
        self.do_transition()
        ret = list()
        try:
            workflow = self.workflow
            transitions = workflow.get_transitions(
                self.model, self.request, from_state=self.model.state)
        except (WorkflowError, AttributeError) as e:
            logger.error("transitions error: %s" % str(e))
            return ret
        workflow_tsf = self.model.workflow_tsf
        for transition in transitions:
            query = make_query(do_transition=transition['name'])
            target = make_url(self.request, node=self.model, query=query)
            props = Properties()
            props.target = target
            if workflow_tsf:
                props.title = workflow_tsf(transition['name'])
            else:
                props.title = transition.get('title', transition['name'])
            ret.append(props)
        return ret



from cone.app.interfaces import IWorkflowState
from plumber import Behavior
from plumber import default
from plumber import override
from plumber import plumb
from pyramid.security import ALL_PERMISSIONS
from pyramid.security import Allow
from pyramid.security import Deny
from pyramid.security import Everyone
from repoze.workflow import get_workflow
from zope.interface import implementer
import logging


logger = logging.getLogger('cone.workflow')


def lookup_workflow(node):
    """Lookup workflow. Original get_workflow expects class if workflow was
    registered for class or instance if workflow was registered for interface.
    """
    workflow_name = node.workflow_name
    workflow = get_workflow(node.__class__, workflow_name)
    if not workflow:
        workflow = get_workflow(node, workflow_name)
    return workflow


def lookup_state_data(node):
    """Lookup state data of current workflow state for node.
    """
    workflow = lookup_workflow(node)
    if not workflow:
        return {}
    return workflow._state_data[node.state]


def initialize_workflow(node, force=False):
    workflow = lookup_workflow(node)
    if not workflow:
        return
    if force or not node.state:
        workflow.initialize(node)


def persist_state(node, info):
    """Transition callback for repoze.workflow.

    Persist state to ``node.state`` and call node.
    """
    node.state = info.transition[u'to_state']
    node()


def permission_checker(permission, node, request):
    """Callback for repoze.workflow to check permissions.

    Necessary since pyramid 1.8, where ``pyramid.security.has_permission``
    has been removed
    """
    return request.has_permission(permission, node)


@implementer(IWorkflowState)
class WorkflowState(Behavior):
    """Behavior for nodes providing workflow states.

    This implementation persists to self.attrs['state']
    """
    workflow_tsf = default(None)
    workflow_name = default(None)

    @plumb
    def __init__(_next, self, *args, **kw):
        _next(self, *args, **kw)
        initialize_workflow(self)

    @plumb
    def copy(_next, self):
        """Set initial state for copied node and all children providing
        ``cone.app.interfaces.IWorkflowState``.
        """
        ret = _next(self)

        def recursiv_initial_state(node):
            if IWorkflowState.providedBy(node):
                initialize_workflow(node, force=True)
                for child in node.values():
                    recursiv_initial_state(child)
        recursiv_initial_state(ret)
        return ret

    @property
    def state(self):
        return self.attrs.get('state', None)

    @default
    @state.setter
    def state(self, val):
        self.attrs['state'] = val


class WorkflowACL(Behavior):
    """Behavior providing ACL's by workflow state.

    Requires ``WorkflowState`` behavior.
    """
    state_acls = default(dict())
    default_acl = default([
        (Allow, 'system.Authenticated', ['view']),
        (Allow, 'role:viewer', ['view']),
        (Allow, 'role:editor', ['view', 'add', 'edit']),
        (Allow, 'role:owner', ['view', 'add', 'edit', 'delete',
                               'change_state', 'manage_permissions']),
        (Allow, 'role:admin', ['view', 'add', 'edit', 'delete',
                               'change_state', 'manage_permissions']),
        (Allow, 'role:manager', ['view', 'add', 'edit', 'delete',
                                 'change_state', 'manage_permissions',
                                 'manage']),
        (Allow, Everyone, ['login']),
        (Deny, Everyone, ALL_PERMISSIONS),
    ])

    @override
    @property
    def __acl__(self):
        acl = self.state_acls.get(self.state, self.default_acl)
        if not acl:
            raise ValueError(u"No ACL found for state '%s'" % self.state)
        if acl is self.default_acl:
            logger.warning(u"No ACL found for state "
                           u"'%s'. Using default" % self.state)
        return acl



from cone.app import compat
from cone.app.browser import RelatedViewConsumer
from cone.app.browser.utils import make_query
from cone.app.browser.utils import make_url
from cone.app.browser.utils import request_property
from cone.app.utils import node_path
from cone.tile import render_template
from cone.tile import Tile
from node.utils import safe_decode
from plumber import plumbing


BATCH_RANGE = 8


class Batch(Tile):
    """An abstract batch tile.

    A subclass has to implement 'self.vocab' and may override
    'self.batchrange', 'self.display' and 'self.batchname'.
    """

    dummypage = {
        'page': '',
        'current': False,
        'visible': False,
        'href': '',
        'target': '',
        'url': '',  # B/C
    }

    path = 'cone.app.browser:templates/batch.pt'
    """Path to batch template.
    """

    ellipsis = u'...'
    """Ellipsis string if number of pages exceeds batch range.
    """

    ajax_path = None
    """Ajax path to set if batch link gets clicked.
    """

    ajax_path_event = None
    """Ajax path event to set if batch link gets clicked.
    """

    trigger_event = 'batchclicked'
    """JS event triggered when page clicked.
    """

    @property
    def trigger_selector(self):
        """CSS selector to trigger JS event to.
        """
        return '.{}sensitiv'.format(self.name)

    @property
    def ajax_event(self):
        return '{}:{}'.format(self.trigger_event, self.trigger_selector)

    @property
    def vocab(self):
        """Batch vocabulary.
        """
        return []

    @property
    def display(self):
        """Flag whether to display the batch.
        """
        return True

    @property
    def batchrange(self):
        """Defines how many pages are displayed.
        """
        return BATCH_RANGE

    @property
    def currentpage(self):
        """Current page in batch.
        """
        for page in self.vocab:
            if page['current']:
                return page
        return None

    @property
    def firstpage(self):
        """First page in batch.
        """
        firstpage = None
        for page in self.vocab:
            if page['visible']:
                firstpage = page
                break
        if not firstpage and self.vocab:
            firstpage = self.vocab[0]
        return firstpage

    @property
    def lastpage(self):
        """Last page in batch.
        """
        lastpage = None
        count = len(self.vocab)
        while count > 0:
            count -= 1
            page = self.vocab[count]
            if page['visible']:
                lastpage = self.vocab[count]
                break
        if not lastpage and self.vocab:
            lastpage = self.vocab[len(self.vocab) - 1]
        return lastpage

    @property
    def prevpage(self):
        """Previous page in batch.
        """
        prevpage = None
        position = self._position_of_current_in_vocab - 1
        while position >= 0:
            page = self.vocab[position]
            if page['visible']:
                prevpage = self.vocab[position]
                break
            position -= 1
        if not prevpage and self.vocab:
            prevpage = self.dummypage
        return prevpage

    @property
    def nextpage(self):
        """Next page in batch.
        """
        nextpage = self.dummypage
        position = self._position_of_current_in_vocab + 1
        if position == 0 and self.vocab:
            return nextpage
        if position == 0 and not self.vocab:
            return None
        while position < len(self.vocab):
            page = self.vocab[position]
            if page['visible']:
                nextpage = self.vocab[position]
                break
            position += 1
        return nextpage

    @property
    def leftellipsis(self):
        """Left ellipsis string.
        """
        return self._left_over_diff < 0 and self.ellipsis or u''

    @property
    def rightellipsis(self):
        """Right ellipsis string.
        """
        return self._right_over_diff < 0 and self.ellipsis or u''

    @property
    def pages(self):
        """Pages to display.
        """
        pos = self._position_of_current_in_vocab
        count = len(self.vocab)
        start = max(pos - self._siderange - max(self._right_over_diff, 0), 0)
        end = min(pos + self._siderange + max(self._left_over_diff, 0) + 1,
                  count)
        return self.vocab[start:end]

    @property
    def _siderange(self):
        return self.batchrange // 2

    @property
    def _left_over_diff(self):
        currentPosition = self._position_of_current_in_vocab
        return self._siderange - currentPosition

    @property
    def _right_over_diff(self):
        position = self._position_of_current_in_vocab
        count = len(self.vocab)
        return position + self._siderange - count + 1

    @property
    def _position_of_current_in_vocab(self):
        # XXX: wildcard handling
        current = self.currentpage
        if current is None:
            return -1
        pointer = 0
        for page in self.vocab:
            if page['page'] == current['page']:
                return pointer
            pointer += 1
        # reached if subclass implementation bug
        return -1


class BatchedItemsBatch(Batch):
    """Displays batched items pagination.
    """

    def __init__(self, parent):
        """Create batched items batch.
        """
        self.parent = parent
        self.name = parent.items_id + 'batch'
        self.ajax_path = parent.ajax_path
        self.ajax_path_event = parent.ajax_path_event

    @property
    def display(self):
        """Flag whether to display the batch.
        """
        return len(self.vocab) > 1

    @property
    def vocab(self):
        """Batch vocabulary.
        """
        ret = list()
        path = node_path(self.model)
        count = self.parent.item_count
        slice_size = self.parent.slice_size
        pages = count // slice_size
        if count % slice_size != 0:
            pages += 1
        current = self.parent.current_page
        for i in range(pages):
            href = self.parent.make_page_url(path, str(i), include_view=True)
            target = self.parent.make_page_url(path, str(i))
            ret.append({
                'page': '%i' % (i + 1),
                'current': current == i,
                'visible': True,
                'href': href,
                'target': target
            })
        return ret


@plumbing(RelatedViewConsumer)
class BatchedItems(Tile):
    """Base tile for displaying searchable, batched items.
    """

    path = 'cone.app.browser:templates/batched_items.pt'
    """Path to template used for rendering the tile. Defaults to
    ``cone.app.browser:templates/batched_items.pt``. Can also be set by passing
    it as ``path`` keyword argument to ``tile`` decorator.
    """

    header_template = 'cone.app.browser:templates/batched_items_header.pt'
    """Template rendering the slice header. Defaults to
    ``cone.app.browser:templates/batched_items_header.pt``.
    """

    footer_template = 'cone.app.browser:templates/batched_items_footer.pt'
    """Template rendering the slice footer. Defaults to
    ``cone.app.browser:templates/batched_items_footer.pt``.
    """

    slice_template = None
    """Template rendering the slice items. Defaults to ``None``. Shall be set
    by subclass.
    """

    items_id = 'batched_items'
    """CSS ID of the batched items container DOM element.
    """

    items_css = 'batched_items panel panel-default'
    """CSS classes of the batched items container DOM element.
    """

    query_whitelist = []
    """Additional incoming request parameters to consider when creating URL's.
    Defaults to ``[]``.
    """

    display_header = True
    """Flag whether to display the listing header. Defaults to ``True``.
    """

    display_footer = True
    """Flag whether to display the listing footer. Defaults to ``True``.
    """

    show_title = True
    """Flag whether to show title in the listing header. Defaults to ``True``.
    """

    title_css = 'col-xs-4'
    """CSS classes to set on title container DOM element.
    Defaults to ``col-xs-4``. Can be used to change the size of the title area.
    """

    default_slice_size = 15
    """Default number of items displayed in slice. Defaults to ``15``.
    """

    num_slice_sizes = 4
    """Number of available slice sizes in slice size selection.
    """

    show_slice_size = True
    """Flag whether to display the slice size selection in listing header.
     Defaults to ``True``.
    """

    slice_size_css = 'col-xs-4 col-sm3'
    """CSS classes to set on slice size selection container DOM element.
    Defaults to ``col-xs-4 col-sm3``. Can be used to change the size of the
    slice size selection.
    """

    show_filter = True
    """Flag whether to display the search filter input in listing header.
    """

    filter_css = 'col-xs-3'
    """CSS classes to set on search filter input container DOM element.
    Defaults to ``col-xs-3``. Can be used to change the size of the search
    filter input.
    """

    head_additional = None
    """Additional arbitrary markup rendered in listing header. Can be used to
    add additional listing filter aspects or similar.
    """

    ajax_path = None
    """Ajax path to set if batched items contents changes.
    """

    ajax_path_event = None
    """Ajax path event to set if items contents changes.
    """

    @property
    def title(self):
        """Batched items title.
        """
        return self.model.metadata.title

    @property
    def bind_selectors(self):
        """CSS selector to bind the batched items container DOM element to.
        """
        return '{}sensitiv'.format(self.pagination.name)

    @property
    def bind_events(self):
        """JS events to bind the batched items container DOM element to.
        """
        return self.pagination.trigger_event

    @property
    def trigger_selector(self):
        """CSS selector to trigger JS event to when changing slice size or
        entering search filter term.
        """
        return self.pagination.trigger_selector

    @property
    def trigger_event(self):
        """JS event triggered when changing slice size or entering search
        filter term.
        """
        return self.pagination.trigger_event

    @property
    def ajax_event(self):
        return '{}:{}'.format(self.trigger_event, self.trigger_selector)

    @property
    def rendered_header(self):
        """Rendered header by ``header_template``.
        """
        if not self.display_header:
            return u''
        return render_template(
            self.header_template,
            request=self.request,
            model=self.model,
            context=self
        )

    @property
    def rendered_footer(self):
        """Rendered footer by ``footer_template``.
        """
        if not self.display_footer:
            return u''
        return render_template(
            self.footer_template,
            request=self.request,
            model=self.model,
            context=self
        )

    @property
    def rendered_slice(self):
        """Rendered slice by ``slice_template``.
        """
        return render_template(
            self.slice_template,
            request=self.request,
            model=self.model,
            context=self
        )

    @property
    def rendered_pagination(self):
        """Rendered pagination batch.
        """
        return self.pagination(
            model=self.model,
            request=self.request
        )

    @request_property
    def pagination(self):
        """``BatchedItemsBatch`` instance.
        """
        return BatchedItemsBatch(parent=self)

    @property
    def current_page(self):
        """Current batch page.
        """
        return int(self.request.params.get('b_page', '0'))

    def make_page_url(self, path, page, include_view=False):
        """Pagination batch page target.
        """
        params = {
            'b_page': page,
            'size': self.slice_size,
            'term': self.filter_term,
        }
        return self.make_url(params, path=path, include_view=include_view)

    # B/C, remove as of cone.app 1.1
    page_target = make_page_url

    @property
    def slice_id(self):
        """CSS ID of the slice container DOM element.
        """
        return '{}_slice'.format(self.items_id)

    @property
    def slice_size(self):
        """Current slice size.
        """
        return int(self.request.params.get('size', self.default_slice_size))

    @property
    def slice_sizes(self):
        """Available slice sices as list.
        """
        return [
            i * self.default_slice_size
            for i in range(1, self.num_slice_sizes + 1)
        ]

    @property
    def current_slice(self):
        """Current slice as (start, end) tuple.
        """
        start = self.current_page * self.slice_size
        end = start + self.slice_size
        return start, end

    @property
    def slice_target(self):
        """Slice size selection target URL.
        """
        return self.make_url({
            'term': self.filter_term,
        })

    @request_property
    def filter_term(self):
        """Current search filter term.
        """
        term = self.request.params.get('term')
        if term:
            term = term.encode('utf-8') if compat.IS_PY2 else term
            term = compat.unquote(term)
            term = term.decode('utf-8') if compat.IS_PY2 else term
        return term

    @property
    def filter_target(self):
        """Search filter input target URL.
        """
        return self.make_url({
            'size': self.slice_size,
        })

    def make_query(self, params):
        """Create query considering ``query_whitelist``.

        :param params: Dictionary with query parameters.
        :return: Query as string.
        """
        p = dict()
        for param in self.query_whitelist:
            p[param] = self.request.params.get(param, '')
        p.update(params)
        return make_query(**p)

    def make_url(self, params, path=None, include_view=False):
        """Create URL considering ``query_whitelist``.

        :param params: Dictionary with query parameters.
        :param path: Optional model path, if ``None``, path gets taken from
            ``self.model``
        :param include_view: Boolean whether to include
            ``self.related_view`` to URL.
        :return: URL as string.
        """
        return safe_decode(make_url(
            self.request,
            path=path,
            node=None if path else self.model,
            resource=self.related_view if include_view else None,
            query=self.make_query(params)))

    @property
    def item_count(self):
        """Overall slice items count.
        """
        raise NotImplementedError(
            "Abstract ``BatchedItems`` does not implement ``item_count``")

    @property
    def slice_items(self):
        """Current slice items.
        """
        raise NotImplementedError(
            "Abstract ``BatchedItems`` does not implement ``items``")



from cone.app.browser.ajax import AjaxAction
from cone.app.browser.ajax import ajax_form_fiddle
from cone.app.browser.utils import format_traceback
from cone.app.browser.utils import make_url
from cone.app.model import AppSettings
from cone.tile import Tile
from cone.tile import render_tile
from cone.tile import tile
from plumber import Behavior
from plumber import default
from plumber import plumb
from pyramid.i18n import TranslationStringFactory
from pyramid.i18n import get_localizer
from pyramid.view import view_config
from webob import Response


_ = TranslationStringFactory('cone.app')


@view_config(name='settings_tab_content', xhr=True, permission='manage')
def settings_tab_content(model, request):
    """Used by jquerytools tabs plugin to get settings section content.
    """
    try:
        rendered = render_tile(model, request, 'content')
    except Exception:
        localizer = get_localizer(request)
        error = localizer.translate(_('error', default='Error'))
        rendered = '<div>{}: {}</div>'.format(error, format_traceback())
    return Response('<div class="{}">{}</div>'.format(model.name, rendered))


@tile(name='content',
      path='templates/settings.pt',
      interface=AppSettings,
      permission='manage')
class AppSettings(Tile):

    @property
    def tabs(self):
        ret = list()
        for val in self.model.values():
            ret.append({
                'title': val.metadata.title,
                'target': make_url(
                    self.request,
                    node=val,
                    resource='settings_tab_content'),
            })
        return ret


class SettingsBehavior(Behavior):
    """Particular settings object form behavior.
    """

    @plumb
    def prepare(_next, self):
        _next(self)
        selector = '#form-{}'.format('-'.join(self.form.path))
        ajax_form_fiddle(self.request, selector, 'replace')

    @default
    def next(self, request):
        url = make_url(request.request, node=self.model)
        selector = '.{}'.format(self.model.name)
        return [AjaxAction(url, 'content', 'inner', selector)]



from cone.app import get_root
from cone.app import security
from cone.app import testing
from cone.app.browser.login import login_view
from cone.app.browser.login import logout_view
from cone.tile import render_tile
from cone.tile.tests import TileTestCase
from webob.response import Response
from webob.exc import HTTPFound


class TestBrowserLogin(TileTestCase):
    layer = testing.security

    def test_login_view(self):
        root = get_root()
        request = self.layer.new_request()
        response = login_view(root, request)
        self.assertTrue(isinstance(response, Response))

    def test_logout_view(self):
        root = get_root()
        request = self.layer.new_request()
        response = logout_view(root, request)
        self.assertTrue(isinstance(response, HTTPFound))

    def test_logout_tile(self):
        root = get_root()
        request = self.layer.new_request()
        with self.layer.authenticated('admin'):
            render_tile(root, request, 'logout')
            self.checkOutput("""
            ResponseHeaders([('Set-Cookie', 'auth_tkt=; Max-Age=0; Path=/; expires=...'),
            ('Set-Cookie', 'auth_tkt=; Domain=example.com; Max-Age=0; Path=/; expires=...'),
            ('Set-Cookie', 'auth_tkt=; Domain=.example.com; Max-Age=0; Path=/; expires=...')])
            """, str(request.response.headers))

    def test_login_form(self):
        root = get_root()
        request = self.layer.new_request()
        res = render_tile(root, request, 'loginform')
        self.assertTrue(res.find('<form action="http://example.com/login"') > -1)

        # Authenticate with wrong credentials
        request.params['loginform.user'] = 'foo'
        request.params['loginform.password'] = 'bar'
        request.params['action.loginform.login'] = '1'
        res = render_tile(root, request, 'loginform')
        self.assertTrue(res.find('class="errormessage">Invalid Credentials') > -1)

        # Authenticate with correct credentials
        request.params['loginform.user'] = security.ADMIN_USER
        request.params['loginform.password'] = security.ADMIN_PASSWORD
        request.params['action.loginform.login'] = '1'
        render_tile(root, request, 'loginform')
        self.assertTrue(isinstance(request.environ['redirect'], HTTPFound))



from cone.app.browser.ajax import AjaxAction
from cone.app.browser.ajax import AjaxEvent
from cone.app.browser.utils import make_url
from cone.tile import Tile
from plumber import Behavior
from plumber import default
from plumber import override
from webob.exc import HTTPFound
from yafowil.controller import Controller
import logging


logger = logging.getLogger('cone.app')


try:
    from yafowil.yaml import parse_from_YAML
except ImportError:  # pragma: no cover
    logger.warning(
        '``yafowil.yaml`` not present. '
        '``cone.app.browser.form.YAMLForm`` will not work')


class YAMLForm(Behavior):
    """Plumbing behavior for rendering yaml forms.
    """
    action_resource = default(u'')

    # B/C
    form_template_path = default(None)

    # use form_template for pointing yaml files
    form_template = default(None)

    # considered in form_action, either 'add' or 'edit'
    form_flavor = default('edit')

    @default
    @property
    def message_factory(self):
        return None

    @default
    def form_action(self, widget, data):
        resource = self.action_resource
        if self.form_flavor == 'add':
            return make_url(self.request, node=self.model.parent,
                            resource=resource)
        return make_url(self.request, node=self.model, resource=resource)

    @override
    def prepare(self):
        if self.form_template:
            self.form = parse_from_YAML(
                self.form_template, self, self.message_factory)
            return
        # BBB
        self.form = parse_from_YAML(
            self.form_template_path, self, self.message_factory)


class ProtectedAttributesForm(Behavior):
    """Plumbing behavior supposed to be used for yafowil forms calculating
    widget modes based on security checks.

    Security declarations for attributes are stored at
    ``self.attribute_permissions`` containing the attribute names as key, and
    a 2-tuple containing required edit and view permission for this attribute.
    """
    attribute_permissions = default(dict())
    attribute_default_permissions = default(('edit', 'view'))

    @default
    def mode_for(self, name):
        """Calculate mode by checking permission defined in
        ``self.attribute_permissions`` for attribute ``name`` against model.

        If no permissions defined for attribute name,
        ``self.attribute_default_permissions`` is used.
        """
        permissions = self.attribute_permissions.get(name)
        if not permissions:
            permissions = self.attribute_default_permissions
        if self.request.has_permission(permissions[0], self.model):
            return 'edit'
        if self.request.has_permission(permissions[1], self.model):
            return 'display'
        return 'skip'


class Form(Tile):
    """A form tile.
    """
    form = None  # yafowil compound expected.
    ajax = True  # render ajax form related by default.

    def prepare(self):
        """Responsible to prepare ``self.form``.
        """
        raise NotImplementedError(u"``prepare`` function must be provided "
                                  u"by deriving object.")

    def prepare_ajax(self):
        """Set ajax class attribute on self.form.
        """
        if not self.ajax:
            return
        if self.form.attrs.get('class_add') \
                and self.form.attrs['class_add'].find('ajax') == -1:
            self.form.attrs['class_add'] += ' ajax'
        else:
            self.form.attrs['class_add'] = 'ajax'

    @property
    def ajax_request(self):
        """Flag whether to handle current request as ajax request.
        """
        return self.request.params.get('ajax') and self.ajax

    def __call__(self, model, request):
        self.model = model
        self.request = request
        return self._process_form()

    def _process_form(self):
        self.prepare()
        self.prepare_ajax()
        if not self.show:
            return u''
        controller = Controller(self.form, self.request)
        if not controller.next:
            return controller.rendered
        if isinstance(controller.next, HTTPFound):
            self.redirect(controller.next)
            return u''
        if isinstance(controller.next, AjaxAction) \
                or isinstance(controller.next, AjaxEvent):
            self.request.environ['cone.app.continuation'] = [controller.next]
            return u''
        if isinstance(controller.next, list):
            # assume a list of ajax continuation definitions
            self.request.environ['cone.app.continuation'] = controller.next
            return u''
        # assume some rendered markup
        return controller.next



from cone.app import compat
from cone.app.browser.ajax import ajax_continue
from cone.app.browser.ajax import ajax_message
from cone.app.browser.ajax import AjaxEvent
from cone.app.browser.utils import choose_name
from cone.app.browser.utils import make_query
from cone.app.browser.utils import make_url
from cone.tile import Tile
from cone.tile import tile
from node.utils import LocationIterator
from pyramid.i18n import get_localizer
from pyramid.i18n import TranslationStringFactory


_ = TranslationStringFactory('cone.app')


def cookie_name(name):
    return u'cone.app.copysupport.%s' % name


def extract_copysupport_cookie(request, name):
    cookie = request.cookies.get(cookie_name(name), '')
    cookie = compat.unquote(cookie)
    paths = cookie.split('::')
    return [path for path in paths if path]


def paths_from_urls(urls):
    ret = list()
    for url in urls:
        splitted = compat.urlparse.urlparse(url).path.split('/')
        ret.append([part for part in splitted if part])
    return ret


@tile(name='paste', permission="paste")
class PasteAction(Tile):

    def render(self):
        cut = extract_copysupport_cookie(self.request, 'cut')
        copy = extract_copysupport_cookie(self.request, 'copy')
        localizer = get_localizer(self.request)
        if not cut and not copy:
            message = localizer.translate(
                _('nothing_to_paste', default='Nothing to paste'))
            ajax_message(self.request, message)
            return u''
        urls = copy and copy or cut
        paths = paths_from_urls(urls)
        call_sources = set()
        errors = list()
        success = 0
        for path in paths:
            node = self.model.root
            for key in path:
                node = node[key]
            if not node.node_info_name:
                message = localizer.translate(
                    _(
                        'cannot_paste_unknown_source',
                        default="Cannot paste '${name}'. Unknown source"
                    ),
                    mapping={'name': node.name}
                )
                errors.append(message)
                continue
            if not self.model.node_info_name:
                message = localizer.translate(
                    _(
                        'cannot_paste_unknown_target',
                        default="Cannot paste to '${name}'. Unknown target"
                    ),
                    mapping={'name': self.model.name}
                )
                errors.append(message)
                continue
            if node.node_info_name not in self.model.nodeinfo.addables:
                message = localizer.translate(
                    _(
                        'cannot_paste_cardinality_violation',
                        default=(
                            "Violation. '${target}' is not allowed to "
                            "contain '${source}'"
                        )
                    ),
                    mapping={
                        'target': self.model.nodeinfo.title,
                        'source': node.nodeinfo.title
                    }
                )
                errors.append(message)
                continue
            source = node.parent
            if copy:
                node = source[node.name].deepcopy()
            else:
                in_model = False
                for parent in LocationIterator(self.model):
                    if parent is node:
                        message = localizer.translate(
                            _(
                                'cannot_paste_self_containment',
                                default=(
                                    "Cannot paste cut object to child "
                                    "of it: ${name}"
                                )
                            ),
                            mapping={'name': parent.name}
                        )
                        errors.append(message)
                        in_model = True
                        break
                if in_model:
                    continue
                node = source.detach(node.name)
            node.__parent__ = self.model
            self.model[choose_name(self.model, node.name)] = node
            if cut:
                call_sources.add(source)
            success += 1
        if success > 0:
            self.model()
        for source in call_sources:
            source()
        message = localizer.translate(
            _(
                'pasted_items',
                default="Pasted ${count} items"
            ),
            mapping={'count': success}
        )
        if errors:
            failed = localizer.translate(
                _(
                    'pasting_items_failed',
                    default="Pasting of ${count} items failed"
                ),
                mapping={'count': len(errors)}
            )
            failed = "<br /><strong>%s</strong>" % failed
            message += "<br />".join([failed] + errors)
        ajax_message(self.request, message)
        content_tile = self.model.properties.action_paste_tile
        if not content_tile:
            content_tile = 'listing'
        query = make_query(contenttile=content_tile)
        url = make_url(self.request, node=self.model, query=query)
        event = AjaxEvent(url, 'contextchanged', '#layout')
        ajax_continue(self.request, event)
        res = self.request.response
        res.delete_cookie(cookie_name(copy and 'copy' or 'cut'))
        return u''



try:  # pragma: no cover
    from StringIO import StringIO
    from urllib2 import quote
    from urllib2 import unquote
    import ConfigParser as configparser
    import urlparse
except ImportError:  # pragma: no cover
    from io import StringIO  # noqa
    from urllib.parse import quote  # noqa
    from urllib.parse import unquote  # noqa
    import configparser  # noqa
    import urllib.parse as urlparse  # noqa
import sys
import types


IS_PY2 = sys.version_info[0] < 3
STR_TYPE = basestring if IS_PY2 else str
UNICODE_TYPE = unicode if IS_PY2 else str
ITER_TYPES = (types.ListType, types.TupleType) if IS_PY2 else (list, tuple)
NUMBER_TYPES = (types.IntType, types.FloatType) if IS_PY2 else (int, float)



from cone.app import cfg
from cone.app import DefaultLayoutConfig
from cone.app import layout_config
from cone.app import testing
from cone.app.browser import render_main_template
from cone.app.browser.actions import LinkAction
from cone.app.browser.ajax import AjaxEvent
from cone.app.browser.layout import LanguageTile
from cone.app.browser.layout import LayoutConfigTile
from cone.app.browser.layout import NavTree
from cone.app.browser.layout import personal_tools
from cone.app.browser.layout import personal_tools_action
from cone.app.browser.layout import ProtectedContentTile
from cone.app.interfaces import ILayoutConfig
from cone.app.interfaces import INavigationLeaf
from cone.app.model import AppRoot
from cone.app.model import BaseNode
from cone.app.model import LayoutConfig
from cone.app.security import DEFAULT_SETTINGS_ACL
from cone.app.testing.mock import default_layout
from cone.app.testing.mock import LayoutConfigNode
from cone.app.testing.mock import WorkflowNode
from cone.tile import render_tile
from cone.tile import Tile
from cone.tile import tile
from cone.tile.tests import TileTestCase
from datetime import datetime
from node.base import BaseNode as NodeBaseNode
from pyramid.security import ALL_PERMISSIONS
from pyramid.security import Deny
from pyramid.security import Everyone
from zope.interface import implementer
import cone.app
import cone.app.browser.login


class TestBrowserLayout(TileTestCase):
    layer = testing.security

    def test_render_main_template(self):
        # To change the default layout, change the main template
        self.checkOutput('...', cone.app.cfg.main_template)

        main = 'cone.app.testing:dummy_main.pt'
        cone.app.cfg.main_template = main

        # An unprotected tile named 'content' registered for all sorts of node
        with self.layer.hook_tile_reg():
            @tile(name='content', permission='login')
            class ContentTile(Tile):
                def render(self):
                    return '<div>Content</div>'

        model = BaseNode()
        request = self.layer.new_request()

        # Render main template. The function accepts an optional ``contenttile``
        # argument. if omitted, reserved name 'content' is used
        res = render_main_template(model, request)
        self.checkOutput("""
        <!DOCTYPE html...<div>Content</div>...</html>
        """, res.text)

        with self.layer.hook_tile_reg():
            @tile(name='othername', permission='login')
            class OtherContentTile(ContentTile):
                def render(self):
                    return '<div>Content</div>'

        res = render_main_template(model, request, contenttile='othername')
        self.checkOutput("""
        <!DOCTYPE html...<div>Content</div>...</html>
        """, res.text)

        # Switch back to default main template
        main = 'cone.app.browser:templates/main.pt'
        cone.app.cfg.main_template = main

        # Non authenticated users only gets unprotected content tile, no
        # controls like navtree, mainmenu, etc
        res = render_main_template(model, request, contenttile='othername')
        self.assertFalse(res.text.find('id="mainmenu"') > -1)
        self.assertFalse(res.text.find('id="navtree"') > -1)
        self.assertFalse(res.text.find('id="personaltools"') > -1)
        self.assertTrue(res.text.find('<div>Content</div>') > -1)

        # Authenticate non privileged
        with self.layer.authenticated('max'):
            res = render_main_template(model, request, contenttile='othername')

        # All tiles protected by 'view' permission are now available to the user
        self.assertTrue(res.text.find('id="mainmenu"') > -1)
        self.assertTrue(res.text.find('id="navtree"') > -1)
        self.assertTrue(res.text.find('id="personaltools"') > -1)

    def test_ProtectedContentTile(self):
        # A login form should be rendered instead of the content for anonymous
        # users. Class ``cone.app.browser.layout.ProtectedContentTile``
        # provides this behavior
        class ProtectedModel(BaseNode):
            pass

        with self.layer.hook_tile_reg():
            @tile(name='content', interface=ProtectedModel, permission='login')
            class ProtectedContent(ProtectedContentTile):
                def render(self):
                    return '<div>Content</div>'

        model = ProtectedModel()
        request = self.layer.new_request()

        # Render protected tile.
        self.checkOutput("""
        <form action="http://example.com/login"
        class="form-horizontal"
        enctype="multipart/form-data" id="form-loginform" method="post"
        novalidate="novalidate">...
        """, render_tile(model, request, 'content'))

        with self.layer.authenticated('max'):
            result = render_tile(model, request, 'content')

        self.assertTrue(result.find('<div>Content</div>') > -1)

        # test coneten permission
        self.assertEqual(ProtectedContent.content_permission, 'view')
        ProtectedContent.content_permission = 'manage'

        with self.layer.authenticated('max'):
            result = render_tile(model, request, 'content')

        self.assertTrue(result.find('<h3>Insufficient privileges</h3>') > -1)

        with self.layer.authenticated('manager'):
            result = render_tile(model, request, 'content')

        self.assertTrue(result.find('<div>Content</div>') > -1)

    def test_mainmenu(self):
        root = BaseNode()
        root['1'] = BaseNode()
        root['2'] = BaseNode()

        request = self.layer.new_request()

        # Render main menu at root unauthorized
        res = render_tile(root, request, 'mainmenu')
        self.assertFalse(res.find('href="http://example.com/1"') > -1)
        self.assertFalse(res.find('href="http://example.com/2"') > -1)

        # Render main menu at root authorized
        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'mainmenu')
        self.assertTrue(res.find('ajax:target="http://example.com/1"') > -1)
        self.assertTrue(res.find('ajax:target="http://example.com/2"') > -1)
        self.assertTrue(res.find('href="http://example.com/1"') > -1)
        self.assertTrue(res.find('href="http://example.com/2"') > -1)

        # Render main menu at child. Child is marked selected
        with self.layer.authenticated('max'):
            res = render_tile(root['1'], request, 'mainmenu')
        self.assertTrue(res.find('<li class="active node-1">') > -1)

        # Render main menu with default child
        model = BaseNode()
        model['1'] = BaseNode()
        model['2'] = BaseNode()
        model.properties.default_child = '2'
        with self.layer.authenticated('max'):
            res = render_tile(model, request, 'mainmenu')
        self.assertTrue(res.find('<li class="active node-2">') > -1)

        # Render main menu on child '1' and check if '2' is unselected now
        with self.layer.authenticated('max'):
            res = render_tile(model['1'], request, 'mainmenu')
        self.assertFalse(res.find('<li class="active node-2">') > -1)
        self.assertTrue(res.find('<li class="active node-1">') > -1)

        # Check rendering of main menu with empty title. This is needed if main
        # menu items are supposed to be displayed as icons via CSS
        with self.layer.authenticated('max'):
            res = render_tile(model, request, 'mainmenu')
        expected = '<span></span></a>'
        self.assertFalse(res.find(expected) > -1)

        model.properties.mainmenu_empty_title = True
        with self.layer.authenticated('max'):
            res = render_tile(model, request, 'mainmenu')
        expected = '<span></span></a>'
        self.assertTrue(res.find(expected) > -1)

        # Child nodes which do not grant permission 'view' are skipped
        class SettingsNode(BaseNode):
            __acl__ = DEFAULT_SETTINGS_ACL

        model['3'] = SettingsNode()
        with self.layer.authenticated('max'):
            res = render_tile(model, request, 'mainmenu')
        self.assertFalse(res.find('<li class=" node-3">') > -1)

        with self.layer.authenticated('manager'):
            res = render_tile(model, request, 'mainmenu')
        self.assertTrue(res.find('<li class=" node-3">') > -1)

        # Check mainmenu displays children
        model = BaseNode()
        child = model['child'] = BaseNode()
        child.properties.mainmenu_display_children = True
        child['1'] = BaseNode()
        child['2'] = BaseNode()
        child['3'] = BaseNode()
        child['3'].properties.skip_mainmenu = True

        with self.layer.authenticated('max'):
            res = render_tile(model, request, 'mainmenu')
        expected = '<li class="dropdown node-child">'
        self.assertTrue(res.find(expected) > -1)
        expected = 'href="http://example.com/child/1"'
        self.assertTrue(res.find(expected) > -1)
        expected = 'href="http://example.com/child/2"'
        self.assertTrue(res.find(expected) > -1)
        expected = 'href="http://example.com/child/3"'
        self.assertFalse(res.find(expected) > -1)

        child['1']['1'] = BaseNode()
        with self.layer.authenticated('max'):
            res = render_tile(child['1']['1'], request, 'mainmenu')
        self.checkOutput("""
        ...<li class="active">
        <a href="http://example.com/child/1"...
        """, res)

    def test_navtree(self):
        root = BaseNode()
        request = self.layer.new_request()

        # Unauthorized
        res = render_tile(root, request, 'navtree')
        self.assertFalse(res.find('id="navtree"') != -1)

        # Empty navtree, no items are marked to be displayed
        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'navtree')
        self.assertTrue(res.find('id="navtree"') != -1)
        self.assertTrue(res.find('ajax:bind="contextchanged"') != -1)
        self.assertTrue(res.find('ajax:action="navtree:#navtree:replace"') != -1)
        self.assertTrue(res.find('class="contextsensitiv list-group"') != -1)

        # Node's which are in navtree
        root = BaseNode()
        root.properties.in_navtree = True
        root['1'] = BaseNode()
        root['1'].properties.in_navtree = True
        root['1']['11'] = BaseNode()
        root['1']['11'].properties.in_navtree = True
        root['1']['11']['111'] = BaseNode()
        root['1']['11']['111'].properties.in_navtree = True
        root['2'] = BaseNode()
        root['2'].properties.in_navtree = True

        # ``in_navtree`` is read from ``node.properties`` and defines display
        # UI contract with the navtree tile
        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'navtree')
        self.assertTrue(res.find('ajax:target="http://example.com/1"') > -1)

        # Render navtree on ``root['1']``, must be selected
        with self.layer.authenticated('max'):
            res = render_tile(root['1'], request, 'navtree')
        self.checkOutput("""
        ...<li class="active navtreelevel_1">
        <a href="http://example.com/1"...
        """, res)

        # Render navtree on ``root['1']['11']``, must be selected
        with self.layer.authenticated('max'):
            res = render_tile(root['1']['11'], request, 'navtree')
        self.checkOutput("""
        ...<li class="active navtreelevel_2">
        <a href="http://example.com/1/11"...
        """, res)

        # Child nodes which not provide IApplicationNode are skipped
        root['3'] = NodeBaseNode()
        with self.layer.authenticated('manager'):
            res = render_tile(root, request, 'navtree')
        self.assertFalse(res.find('ajax:target="http://example.com/3"') > -1)

        # Subtree rendering stops if node provides INaviationLeaf
        @implementer(INavigationLeaf)
        class LeafNode(BaseNode):
            pass

        root['3'] = LeafNode()
        root['3'].properties.in_navtree = True
        root['3']['3'] = BaseNode()
        root['3']['3'].properties.in_navtree = True

        with self.layer.authenticated('manager'):
            res = render_tile(root['3'], request, 'navtree')
        self.assertTrue(res.find('ajax:target="http://example.com/3"') > -1)
        self.assertFalse(res.find('ajax:target="http://example.com/3/3"') > -1)

        # Child nodes which do not grant permission 'view' are skipped
        class InvisibleNavNode(BaseNode):
            __acl__ = DEFAULT_SETTINGS_ACL

        root['3'] = InvisibleNavNode()
        root['3'].properties.in_navtree = True
        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'navtree')
        self.assertFalse(res.find('ajax:target="http://example.com/3"') > -1)

        with self.layer.authenticated('manager'):
            res = render_tile(root, request, 'navtree')
        self.assertTrue(res.find('ajax:target="http://example.com/3"') > -1)

        # Workflow state
        root['4'] = WorkflowNode()
        with self.layer.authenticated('manager'):
            res = render_tile(root, request, 'navtree')
        self.checkOutput("""
        ...<li class="state-initial navtreelevel_1">
        <a href="http://example.com/4"...
        """, res)

        # Default child behavior of navtree. Default children objects are
        # displayed in navtree.
        root.properties.default_child = '1'
        with self.layer.authenticated('manager'):
            res = render_tile(root, request, 'navtree')
        self.checkOutput("""
        ...<li class="active navtreelevel_1">
        <a href="http://example.com/1"...
        """, res)

        with self.layer.authenticated('manager'):
            res = render_tile(root['1'], request, 'navtree')
        self.checkOutput("""
        ...<li class="active navtreelevel_1">
        <a href="http://example.com/1"...
        """, res)

        # If default child should not be displayed it navtree,
        # ``node.properties.hide_if_default`` must be set to 'True'
        root['1'].properties.hide_if_default = True

        # In this case, also children context gets switched. Instead of
        # remaining non default children, children of default node are
        # displayed.
        with self.layer.authenticated('manager'):
            res = render_tile(root, request, 'navtree')
        self.assertFalse(res.find('ajax:target="http://example.com/1"') > -1)
        self.assertFalse(res.find('ajax:target="http://example.com/2"') > -1)
        self.assertTrue(res.find('ajax:target="http://example.com/1/11"') > -1)

        # Check whether children subrendering works on nodes which have set
        # ``hide_if_default``
        root['1']['11']['a'] = BaseNode()
        root['1']['11']['a'].properties.in_navtree = True
        root['1']['11']['a']['aa'] = BaseNode()
        root['1']['11']['a']['aa'].properties.in_navtree = True
        root['1']['11']['b'] = BaseNode()
        root['1']['11']['b'].properties.in_navtree = True

        with self.layer.authenticated('manager'):
            res = render_tile(root['1']['11'], request, 'navtree')
        self.assertTrue(res.find('ajax:target="http://example.com/1/11/a"') > -1)
        self.assertTrue(res.find('ajax:target="http://example.com/1/11/b"') > -1)

        with self.layer.authenticated('manager'):
            res = render_tile(root['1']['11']['a'], request, 'navtree')
        self.assertTrue(res.find('ajax:target="http://example.com/1/11/a/aa"') > -1)

        with self.layer.authenticated('manager'):
            res = render_tile(root['1']['11']['a']['aa'], request, 'navtree')
        self.assertTrue(res.find('ajax:target="http://example.com/1/11/a/aa"') > -1)

        # Render navtree on ``root['1']['11']``, check selected
        with self.layer.authenticated('manager'):
            res = render_tile(root['1']['11'], request, 'navtree')
        self.checkOutput("""
        ...<li class="active navtreelevel_1">
        <a href="http://example.com/1/11"...
        """, res)

        # Nodes can be marked as navigation root
        class TestNavTree(NavTree):
            def __init__(self, model, request):
                self.model = model
                self.request = request

        ignored_root = BaseNode(name='ignored_root')
        ignored_root.properties.in_navtree = True
        navroot = ignored_root['navroot'] = BaseNode()
        navroot.properties.in_navtree = True
        navroot.properties.is_navroot = True
        navroot.metadata.title = 'Navigation Root'
        navroot['child_1'] = BaseNode()
        navroot['child_1'].properties.in_navtree = True
        navroot['child_2'] = BaseNode()
        navroot['child_2'].properties.in_navtree = True

        navtree = TestNavTree(navroot, request)
        self.assertEqual(navtree.navroot.name, 'navroot')
        self.assertEqual(navtree.title, 'Navigation Root')

        with self.layer.authenticated('manager'):
            self.assertEqual(len(navtree.navtree()['children']), 2)

    def test_personal_tools_action(self):
        @personal_tools_action(name='testaction')
        class TestAction(LinkAction):
            pass

        self.assertTrue('testaction' in personal_tools)
        self.assertIsInstance(personal_tools['testaction'], TestAction)

        del personal_tools['testaction']

    def test_personaltools(self):
        root = BaseNode()
        request = self.layer.new_request()

        # Unauthorized
        res = render_tile(root, request, 'personaltools')
        self.assertFalse(res.find('id="personaltools"') > -1)

        # Authorized
        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'personaltools')
        self.assertTrue(res.find('id="personaltools"') > -1)
        self.assertTrue(res.find('href="http://example.com/logout"') > -1)
        self.assertFalse(res.find('href="http://example.com/settings"') > -1)

        # No settings link if empty settings
        root['settings'] = BaseNode()
        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'personaltools')
        self.assertFalse(res.find('href="http://example.com/settings"') > -1)

        # Settings link if settings container contains children
        root['settings']['mysettings'] = BaseNode()
        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'personaltools')
        self.assertTrue(res.find('href="http://example.com/settings"') > -1)

        # No settings if no view permission
        class NoAccessSettings(BaseNode):
            __acl__ = [(Deny, Everyone, ALL_PERMISSIONS)]

        root['settings'] = NoAccessSettings()
        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'personaltools')
        self.assertFalse(res.find('href="http://example.com/settings"') > -1)

    def test_pathbar(self):
        root = BaseNode()
        root['1'] = BaseNode()
        request = self.layer.new_request()

        # Unauthorized
        res = render_tile(root, request, 'pathbar')
        self.assertFalse(res.find('pathbaritem') != -1)

        # Authorized
        with self.layer.authenticated('max'):
            res = render_tile(root['1'], request, 'pathbar')
        self.assertTrue(res.find('id="pathbar"') != -1)

        # Default child behavior of pathbar
        root = BaseNode()
        root['1'] = BaseNode()
        root['2'] = BaseNode()

        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'pathbar')
        self.assertTrue(res.find('<strong>Home</strong>') > -1)

        with self.layer.authenticated('max'):
            res = render_tile(root['1'], request, 'pathbar')
        self.assertTrue(res.find('>Home</a>') > -1)
        self.assertTrue(res.find('<strong>1</strong>') > -1)

        with self.layer.authenticated('max'):
            res = render_tile(root['2'], request, 'pathbar')
        self.assertTrue(res.find('>Home</a>') > -1)
        self.assertTrue(res.find('<strong>2</strong>') > -1)

        root.properties.default_child = '1'
        with self.layer.authenticated('max'):
            res = render_tile(root['1'], request, 'pathbar')
        self.assertTrue(res.find('<strong>Home</strong>') > -1)
        self.assertFalse(res.find('<strong>1</strong>') > -1)

        with self.layer.authenticated('max'):
            res = render_tile(root['2'], request, 'pathbar')
        self.assertTrue(res.find('>Home</a>') > -1)
        self.assertTrue(res.find('<strong>2</strong>') > -1)

        root['1'].properties.default_child = '12'
        root['1']['11'] = BaseNode()
        root['1']['12'] = BaseNode()

        with self.layer.authenticated('max'):
            res = render_tile(root['1']['11'], request, 'pathbar')
        self.assertTrue(res.find('<strong>11</strong>') > -1)

        with self.layer.authenticated('max'):
            res = render_tile(root['1']['12'], request, 'pathbar')
        self.assertTrue(res.find('<strong>Home</strong>') > -1)

    def test_byline(self):
        # Byline renders ``model.metadata.creator``, ``model.metadata.created``
        # and ``model.metadata.modified``
        dt = datetime(2011, 3, 14)
        root = BaseNode()
        root.metadata.created = dt
        root.metadata.modified = dt
        root.metadata.creator = 'max'
        request = self.layer.new_request()

        # Unauthenticated
        res = render_tile(root, request, 'byline')
        self.assertEqual(res, u'')

        # Authenticated
        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'byline')
        self.checkOutput("""
        <p class="byline">
          <span>Created by</span>:
          <strong>max</strong>,
          <span>on</span>
          <strong>14.03.2011 00:00</strong>.
          <span>Last modified</span>:
          <strong>14.03.2011 00:00</strong>
        </p>
        """, res)

    def test_default_root_content(self):
        # Default root
        root = AppRoot()
        request = self.layer.new_request()
        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'content')
        self.assertEqual(res, '<div>Default Root</div>')

        # Default child
        class DefaultChild(BaseNode):
            pass

        with self.layer.hook_tile_reg():
            @tile(name='content', interface=DefaultChild, permission='view')
            class DefaultChildContentTile(Tile):
                def render(self):
                    return '<div>Default Child Content</div>'

        root = AppRoot()
        root.factories['1'] = DefaultChild
        root.properties.default_child = '1'
        request = self.layer.new_request()
        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'content')
        self.assertEqual(res, '<div>Default Child Content</div>')

        # Default content tile
        with self.layer.hook_tile_reg():
            @tile(name='mycontent', interface=AppRoot, permission='view')
            class MyRootContentTile(Tile):
                def render(self):
                    return '<div>My Root Content Tile</div>'

        root = AppRoot()
        root.properties.default_content_tile = 'mycontent'
        request = self.layer.new_request()
        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'content')
        self.assertEqual(res, '<div>My Root Content Tile</div>')

        # Custom root content tile
        with self.layer.hook_tile_reg():
            @tile(name='content', interface=AppRoot, permission='view')
            class RootContentTile(Tile):
                def render(self):
                    return '<div>Root Content</div>'

        root = AppRoot()
        request = self.layer.new_request()
        with self.layer.authenticated('max'):
            res = render_tile(root, request, 'content')
        self.assertEqual(res, '<div>Root Content</div>')

    def test_layout_config_tile(self):
        # B/C from layout attribute
        tile = LayoutConfigTile()
        tile.model = LayoutConfigNode()
        tile.request = self.layer.new_request()
        config = tile.config
        self.assertIsInstance(config, LayoutConfig)

        # B/C from layout adapter
        request = self.layer.new_request()
        request.registry.registerAdapter(default_layout)

        model = BaseNode()
        config = request.registry.queryAdapter(model, ILayoutConfig, default=None)
        self.assertIsInstance(config, LayoutConfig)

        tile = LayoutConfigTile()
        tile.model = model
        tile.request = request
        config = tile.config
        self.assertIsInstance(config, LayoutConfig)

        request.registry.unregisterAdapter(default_layout)
        config = request.registry.queryAdapter(model, ILayoutConfig, default=None)
        self.assertEqual(config, None)

        # from layout_config
        tile = LayoutConfigTile()
        tile.model = BaseNode()
        tile.request = self.layer.new_request()
        config = tile.config
        self.assertIsInstance(config, DefaultLayoutConfig)

        # default child
        class ChildNode(BaseNode):
            pass

        @layout_config(ChildNode)
        class ChildNodeLayout(LayoutConfig):
            pass

        model = BaseNode()
        model['child'] = ChildNode()
        model.properties.default_child = 'child'
        tile = LayoutConfigTile()
        tile.model = model
        tile.request = self.layer.new_request()
        config = tile.config
        self.assertIsInstance(config, ChildNodeLayout)

        del layout_config._registry[ChildNode]

    def test_LanguageTile(self):
        tile = LanguageTile()
        request = tile.request = self.layer.new_request()
        self.assertEqual(tile.param_blacklist, [
            '_',
            '_LOCALE_',
            'bdajax.action',
            'bdajax.mode',
            'bdajax.selector'
        ])
        request.params['_'] = '123'
        request.params['existing'] = 'value'
        self.assertEqual(tile.make_query(), '?existing=value')
        self.assertEqual(tile.make_query(lang='en'), '?existing=value&lang=en')

    def test_language(self):
        self.assertEqual(cfg.available_languages, ['en', 'de'])
        root = BaseNode()
        request = self.layer.new_request()

        res = render_tile(root, request, 'language')
        self.checkOutput("""
        <li class="dropdown">
          <a href="#"
             class="dropdown-toggle"
             data-toggle="dropdown">
             <span>Language</span>
             <span class="caret"></span>
          </a>
          <ul class="dropdown-menu" role="languagemenu">
            <li>
              <a href="#"
                 ajax:bind="click"
                 ajax:target="http://example.com/?lang=en"
                 ajax:action="change_language:NONE:NONE">
                 <span class="icon-lang-en"></span>
                English
              </a>
            </li>
            <li>
              <a href="#"
                 ajax:bind="click"
                 ajax:target="http://example.com/?lang=de"
                 ajax:action="change_language:NONE:NONE">
                 <span class="icon-lang-de"></span>
                German
              </a>
            </li>
          </ul>
        </li>
        """, res)

        cfg.available_languages = []
        res = render_tile(root, request, 'language')
        cfg.available_languages = ['en', 'de']
        self.assertEqual(res, '')

    def test_change_language(self):
        self.assertEqual(cfg.available_languages, ['en', 'de'])
        root = BaseNode()
        request = self.layer.new_request()

        request.params['lang'] = 'de'
        render_tile(root, request, 'change_language')

        cookie = request.response.headers['Set-Cookie']
        self.assertTrue(cookie.startswith('_LOCALE_=de;'))

        continuation = request.environ['cone.app.continuation']
        self.assertEqual(len(continuation), 1)

        event = continuation[0]
        self.assertIsInstance(event, AjaxEvent)
        self.assertEqual(event.target, 'http://example.com/')
        self.assertEqual(event.name, 'contextchanged')
        self.assertEqual(event.selector, '#layout')



from cone.app import testing
from cone.app.browser.ajax import ajax_tile
from cone.app.browser.sharing import sharing
from cone.app.model import BaseNode
from cone.app.testing.mock import SharingNode
from cone.tile import render_tile
from cone.tile.tests import TileTestCase
from pyramid.exceptions import HTTPForbidden


class TestBrowserSharing(TileTestCase):
    layer = testing.security

    def test_render_sharing_view(self):
        model = SharingNode(name='root')
        request = self.layer.new_request()

        err = self.expectError(
            HTTPForbidden,
            sharing,
            model,
            request
        )
        self.checkOutput("""
        ...Unauthorized: tile <cone.app.browser.sharing.SharingTile object at ...>
        failed permission check...
        """, str(err))

        with self.layer.authenticated('manager'):
            res = sharing(model, request)
        self.assertTrue(res.text.find('<!DOCTYPE html>') > -1)

    def test_render_sharing_tile(self):
        root = SharingNode(name='root')
        request = self.layer.new_request()

        # Render sharing tile
        with self.layer.authenticated('manager'):
            res = render_tile(root, request, 'sharing')
        self.checkOutput("""
        ...<table class="table table-striped table-condensed"
        id="localacltable_table">...
        """, res)

    def test_search_principal(self):
        root = SharingNode(name='root')
        request = self.layer.new_request()

        # Render sharing tile with search term
        with self.layer.authenticated('manager'):
            res = render_tile(root, request, 'sharing')
        self.assertFalse(res.find('Manager User') > -1)

        request.params['term'] = 'manager'
        with self.layer.authenticated('manager'):
            res = render_tile(root, request, 'sharing')
        self.assertTrue(res.find('Manager User') > -1)

        request.params['term'] = 'group1'
        with self.layer.authenticated('manager'):
            res = render_tile(root, request, 'sharing')
        self.assertTrue(res.find('Group 1') > -1)

        # Existing principal roles are not rendered if term found on request
        root.principal_roles['viewer'] = ['editor']
        with self.layer.authenticated('manager'):
            res = render_tile(root, request, 'sharing')
        expected = (
            '<input checked="checked" '
            'class="add_remove_role_for_principal" '
            'id="input-viewer" name="viewer" type="checkbox" value="editor" />'
        )
        self.assertFalse(res.find(expected) > -1)

        # Existing principal roles are rendered if no term found
        del request.params['term']
        with self.layer.authenticated('manager'):
            res = render_tile(root, request, 'sharing')
        self.assertTrue(res.find(expected) > -1)

    def test_inherited_principal_roles(self):
        root = SharingNode(name='root')
        root.principal_roles['viewer'] = ['editor']

        child = root['child'] = SharingNode()
        child.role_inheritance = True
        child.principal_roles['viewer'] = ['admin']

        request = self.layer.new_request()

        with self.layer.authenticated('manager'):
            res = render_tile(child, request, 'sharing')

        expected = (
            '<input checked="checked" '
            'class="add_remove_role_for_principal" disabled="disabled" '
            'id="input-viewer" name="viewer" type="checkbox" value="editor" />'
        )
        self.assertTrue(res.find(expected) > -1)

        expected = (
            '<input checked="checked" '
            'class="add_remove_role_for_principal" id="input-viewer" '
            'name="viewer" type="checkbox" value="admin" />'
        )
        self.assertTrue(res.find(expected) > -1)

        expected = (
            '<input class="add_remove_role_for_principal" '
            'id="input-viewer" name="viewer" type="checkbox" '
            'value="manager" />'
        )
        self.assertTrue(res.find(expected) > -1)

    def test_table_sorting(self):
        root = SharingNode(name='root')
        child = root['child'] = SharingNode()
        child.principal_roles['viewer'] = ['admin']
        child.principal_roles['editor'] = ['admin']

        request = self.layer.new_request()

        # Sharing table sorting
        with self.layer.authenticated('manager'):
            res = render_tile(child, request, 'sharing')
        self.assertTrue(res.find('Editor User') > -1)
        self.assertTrue(res.find('Viewer User') > -1)
        self.assertTrue(res.find('Editor User') < res.find('Viewer User'))

        request.params['order'] = 'desc'
        with self.layer.authenticated('manager'):
            res = render_tile(child, request, 'sharing')
        self.assertTrue(res.find('Editor User') > -1)
        self.assertTrue(res.find('Viewer User') > -1)
        self.assertFalse(res.find('Editor User') < res.find('Viewer User'))

        del request.params['order']

    def test_skip_inexistent(self):
        root = SharingNode(name='root')
        child = root['child'] = SharingNode()

        # Users defined in ``principal_roles`` but not exists in ugm are
        # skipped. This could happen if user was deleted but principal roles
        # were not
        child.principal_roles['inexistent'] = ['viewer']

        request = self.layer.new_request()

        with self.layer.authenticated('manager'):
            res = render_tile(child, request, 'sharing')
        self.assertFalse(res.find('name="inexistent"') > -1)

    def test_add_role(self):
        root = SharingNode(name='root')
        child = root['child'] = SharingNode()
        child.principal_roles['viewer'] = ['admin']
        child.principal_roles['editor'] = ['admin']

        # Add role for user
        request = self.layer.new_request()
        request.params['id'] = 'viewer'
        request.params['role'] = 'manager'
        request.params['bdajax.action'] = 'add_principal_role'
        request.params['bdajax.mode'] = 'NONE'
        request.params['bdajax.selector'] = 'NONE'

        # Nothing happens if success
        with self.layer.authenticated('manager'):
            res = ajax_tile(child, request)
        self.assertEqual(res, {
            'continuation': False,
            'payload': u'',
            'mode': 'NONE',
            'selector': 'NONE'
        })

        # Principal roles have changed
        self.assertEqual(len(child.principal_roles), 2)
        self.assertEqual(
            sorted(child.principal_roles['viewer']),
            ['admin', 'manager']
        )
        self.assertEqual(child.principal_roles['editor'], ['admin'])

        # Add role for user not added yet
        request.params['id'] = 'otheruser'
        request.params['role'] = 'manager'
        with self.layer.authenticated('manager'):
            res = ajax_tile(child, request)
        self.assertEqual(res, {
            'continuation': False,
            'payload': u'',
            'mode': 'NONE',
            'selector': 'NONE'
        })

        self.assertEqual(len(child.principal_roles), 3)
        self.assertEqual(
            sorted(child.principal_roles['viewer']),
            ['admin', 'manager']
        )
        self.assertEqual(child.principal_roles['editor'], ['admin'])
        self.assertEqual(child.principal_roles['otheruser'], ['manager'])

        # If an error occurs, a message gets displayed
        invalid_node = BaseNode()
        request.params['id'] = 'viewer'
        with self.layer.authenticated('manager'):
            res = ajax_tile(invalid_node, request)
        self.assertEqual(res, {
            'continuation': [{
                'flavor': 'error',
                'type': 'message',
                'payload': u"Can not add role 'manager' for principal 'viewer'",
                'selector': None
            }],
            'payload': u'',
            'mode': 'NONE',
            'selector': 'NONE'
        })

    def test_remove_role(self):
        root = SharingNode(name='root')
        child = root['child'] = SharingNode()
        child.principal_roles['viewer'] = ['admin', 'manager']
        child.principal_roles['editor'] = ['admin']
        child.principal_roles['otheruser'] = ['manager']

        # Remove role for user
        request = self.layer.new_request()
        request.params['id'] = 'viewer'
        request.params['role'] = 'manager'
        request.params['bdajax.action'] = 'remove_principal_role'
        request.params['bdajax.mode'] = 'NONE'
        request.params['bdajax.selector'] = 'NONE'

        # Nothing happens if success
        with self.layer.authenticated('manager'):
            res = ajax_tile(child, request)
        self.assertEqual(res, {
            'continuation': False,
            'payload': u'',
            'mode': 'NONE',
            'selector': 'NONE'
        })

        # Principal roles has changed
        self.assertEqual(child.principal_roles, {
            'viewer': ['admin'],
            'editor': ['admin'],
            'otheruser': ['manager']
        })

        # Principal id gets removed if no more roles left
        request.params['id'] = 'otheruser'
        request.params['role'] = 'manager'
        with self.layer.authenticated('manager'):
            res = ajax_tile(child, request)
        self.assertEqual(res, {
            'continuation': False,
            'payload': u'',
            'mode': 'NONE',
            'selector': 'NONE'
        })

        self.assertEqual(child.principal_roles, {
            'viewer': ['admin'],
            'editor': ['admin']
        })

        # If an error occurs, a message gets displayed.
        # Inexistent role
        request.params['id'] = 'viewer'
        request.params['role'] = 'inexistent'
        with self.layer.authenticated('manager'):
            res = ajax_tile(child, request)
        self.assertEqual(res, {
            'continuation': [{
                'flavor': 'error',
                'type': 'message',
                'payload': u"Can not remove role 'inexistent' for principal 'viewer'",
                'selector': None
            }],
            'payload': u'',
            'mode': 'NONE',
            'selector': 'NONE'
        })

        # Inexistent userid
        request = self.layer.new_request()
        request.params['id'] = 'foo'
        request.params['role'] = 'manager'
        request.params['bdajax.action'] = 'remove_principal_role'
        request.params['bdajax.mode'] = 'NONE'
        request.params['bdajax.selector'] = 'NONE'
        with self.layer.authenticated('manager'):
            res = ajax_tile(child, request)
        self.assertEqual(res, {
            'continuation': [{
                'flavor': 'error',
                'type': 'message',
                'payload': u"Can not remove role 'manager' for principal 'foo'",
                'selector': None
            }],
            'payload': u'',
            'mode': 'NONE',
            'selector': 'NONE'
        })



from cone.app.browser.actions import ActionContext
from cone.tile import render_template_to_response
from plumber import Behavior
from plumber import default
from plumber import plumb
from pyramid.response import Response
from pyramid.static import static_view
from pyramid.view import view_config
import cone.app
import os
import yafowil.loader  # noqa


static_resources = static_view('static', use_subpath=True)


def render_main_template(model, request, contenttile='content'):
    """Renders main template and return response object.

    As main content the tile with name contenttile is rendered.
    """
    ActionContext(model, request, contenttile)
    return render_template_to_response(
        cone.app.cfg.main_template,
        request=request,
        model=model
    )


@view_config(permission='login')
def main_view(model, request):
    """Default view.
    """
    return render_main_template(model, request)


FAVICON_FILE = os.path.join(os.path.dirname(__file__), 'static', 'favicon.ico')


@view_config(route_name='favicon')
def favicon_view(request):
    response = Response()
    with open(FAVICON_FILE, 'rb') as f:
        response.body = f.read()
    response.headers['Content-Type'] = 'image/vnd.microsoft.icon'
    return response


###############################################################################
# Related view support
###############################################################################

def set_related_view(request, view_name):
    """Store related view name on request.

    :param request: HTTP request instance
    :param view_name: View name related to this HTTP request
    """
    request.environ['cone.app.related_view'] = view_name


def get_related_view(request):
    """Return related view name from request.

    :param request: HTTP request instance
    """
    return request.environ.get('cone.app.related_view', None)


class RelatedViewProvider(Behavior):
    """Plumbing behavior for providing related view name.

    This behavior can be applied on tiles and is supposed to be used to set
    a related view name for the current request.

    This related view name can be used then by nested tiles to create browser
    URLs.
    """

    related_view = default(None)
    """Related view name to set on request.
    """

    @plumb
    def __call__(_next, self, model, request):
        """Set related view on request and call downstream function.
        """
        set_related_view(request, self.related_view)
        return _next(self, model, request)


class RelatedViewConsumer(Behavior):
    """Plumbing behavior for consuming related view name.

    This behavior can be applied on tiles and is supposed to be used on
    generic tiles for considering the current view name when creating browser
    URLs.

    This is useful for proper browser path and history support and for setting
    correct link href while keeping ajax targets sane.
    """

    @default
    @property
    def related_view(self):
        """Return related view name from request.
        """
        return get_related_view(self.request)



from cone.app import testing
from cone.app.ugm import BCFileUGMFactory
from cone.app.ugm import FileUGMFactory
from cone.app.ugm import principal_data
from cone.app.ugm import ugm_backend
from cone.app.ugm import UGMFactory
from node.ext.ugm.file import Ugm as FileUgm
from node.tests import NodeTestCase


def restore_ugm_backend(fn):
    def wrapper(*a, **kw):
        registry = ugm_backend.registry
        name = ugm_backend.name
        factory = ugm_backend.factory
        ugm = ugm_backend.ugm
        try:
            fn(*a, **kw)
        finally:
            ugm_backend.name = name
            ugm_backend.factory = factory
            ugm_backend.ugm = ugm
            ugm_backend.registry = registry
    return wrapper


class TestUgm(NodeTestCase):
    layer = testing.security

    def test_UGMFactory(self):
        self.expect_error(NotImplementedError, UGMFactory, {})

        class DummyUGMFactory(UGMFactory):
            def __init__(self, settings):
                pass

        self.expect_error(NotImplementedError, DummyUGMFactory({}).__call__)

    @restore_ugm_backend
    def test_ugm_backend(self):
        ugm_backend.registry = dict()
        ugm_backend.name = None
        ugm_backend.factory = None
        ugm_backend.ugm = None

        class DummyUGM(object):
            pass

        @ugm_backend('dummy')
        class DummyUGMFactory(UGMFactory):
            def __init__(self, settings):
                pass

            def __call__(self):
                return DummyUGM()

        self.assertEqual(ugm_backend.registry, {'dummy': DummyUGMFactory})

        err = self.expect_error(ValueError, ugm_backend.load, 'inexistent', {})
        self.assertEqual(str(err), 'Unknown UGM backend "inexistent"')

        err = self.expect_error(ValueError, ugm_backend.initialize)
        self.assertEqual(str(err), 'UGM backend not loaded')

        self.assertEqual(ugm_backend.name, None)
        self.assertEqual(ugm_backend.factory, None)
        self.assertEqual(ugm_backend.ugm, None)

        ugm_backend.load('dummy', {})

        self.assertEqual(ugm_backend.name, 'dummy')
        self.assertTrue(isinstance(ugm_backend.factory, DummyUGMFactory))
        self.assertEqual(ugm_backend.ugm, None)

        ugm_backend.initialize()
        self.assertTrue(isinstance(ugm_backend.ugm, DummyUGM))

        ugm = ugm_backend.ugm
        ugm_backend.initialize()
        self.assertFalse(ugm is ugm_backend.ugm)

    @restore_ugm_backend
    def test_FileUGMFactory(self):
        self.assertTrue('file' in ugm_backend.registry)
        self.assertTrue(ugm_backend.registry['file'] is FileUGMFactory)

        ugm_backend.load('file', {
            'ugm.users_file': 'users',
            'ugm.groups_file': 'groups',
            'ugm.roles_file': 'roles',
            'ugm.datadir': 'userdata',
            'ugm.user_expires_attr': 'expires'
        })
        ugm_backend.initialize()
        ugm = ugm_backend.ugm

        self.assertTrue(isinstance(ugm, FileUgm))
        self.assertEqual(ugm.users_file, 'users')
        self.assertEqual(ugm.groups_file, 'groups')
        self.assertEqual(ugm.roles_file, 'roles')
        self.assertEqual(ugm.data_directory, 'userdata')
        self.assertEqual(ugm.user_expires_attr, 'expires')

    @restore_ugm_backend
    def test_BCFileUGMFactory(self):
        self.assertTrue('node.ext.ugm' in ugm_backend.registry)
        self.assertTrue(ugm_backend.registry['node.ext.ugm'] is BCFileUGMFactory)

        ugm_backend.load('node.ext.ugm', {
            'node.ext.ugm.users_file': 'users',
            'node.ext.ugm.groups_file': 'groups',
            'node.ext.ugm.roles_file': 'roles',
            'node.ext.ugm.datadir': 'userdata',
            'node.ext.ugm.user_expires_attr': 'expires'
        })
        ugm_backend.initialize()
        ugm = ugm_backend.ugm

        self.assertTrue(isinstance(ugm, FileUgm))
        self.assertEqual(ugm.users_file, 'users')
        self.assertEqual(ugm.groups_file, 'groups')
        self.assertEqual(ugm.roles_file, 'roles')
        self.assertEqual(ugm.data_directory, 'userdata')
        self.assertEqual(ugm.user_expires_attr, 'expires')

    def test_principal_data(self):
        # Fetch principal data
        self.assertEqual(principal_data('manager').items(), [
            (u'fullname', u'Manager User'),
            (u'email', u'manager@bar.com')
        ])
        self.assertEqual(principal_data('inexistent'), {})

        # If UGM implementation raises an exception when trying to fetch the
        # principal it get logged
        orgin_ugm = ugm_backend.ugm
        ugm_backend.ugm = None

        self.assertEqual(principal_data('inexistent'), {})
        # XXX: check logs

        ugm_backend.ugm = orgin_ugm



from cone.app import testing
from cone.app.testing.mock import InexistentWorkflowNode
from cone.app.testing.mock import WorkflowNode
from cone.tile import render_tile
from cone.tile.tests import TileTestCase
from pyramid.i18n import TranslationStringFactory


class TestBrowserWorkflow(TileTestCase):
    layer = testing.security

    def test_workflow_dropdown(self):
        request = self.layer.new_request()
        node = WorkflowNode()

        self.assertEqual(node.state, u'initial')

        with self.layer.authenticated('manager'):
            res = render_tile(node, request, 'wf_dropdown')
        self.checkOutput("""
        ...<a href="#"
              class="state-initial dropdown-toggle"
              title="Change state"
              data-toggle="dropdown">
          <span>State</span>:
          <span>Initial State</span>
        </a>...
        """, res)

        request.params['do_transition'] = 'initial_2_final'
        with self.layer.authenticated('manager'):
            res = render_tile(node, request, 'wf_dropdown')
        self.checkOutput("""
        ...<li class="dropdown">
          <span class="no-transitions state-final">
            <span>State</span>:
            <span>Final State</span>
          </span>
        </li>...
        """, res)

        self.assertEqual(node.state, u'final')

        node = InexistentWorkflowNode()
        request = self.layer.new_request()
        with self.layer.authenticated('manager'):
            res = render_tile(node, request, 'wf_dropdown')
        self.assertEqual(res, u'\n\n  \n\n\n')

        tsf = TranslationStringFactory('cone.app.tests')
        node = WorkflowNode()
        node.workflow_tsf = tsf

        with self.layer.authenticated('manager'):
            res = render_tile(node, request, 'wf_dropdown')
        self.checkOutput("""
        ...<a href="#"
              class="state-initial dropdown-toggle"
              title="Change state"
              data-toggle="dropdown">
          <span>State</span>:
          <span>initial</span>
        </a>...
        """, res)
